@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2020-2023 NXP
+# Copyright 2020-2025 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
@@ -9,52 +9,47 @@
 
 import inspect
 import json
-import logging
 import sys
 from typing import Optional
 
 import click
 
+from spsdk.apps.utils import spsdk_logger
 from spsdk.apps.utils.common_cli_options import (
     CommandsTreeGroup,
-    isp_interfaces,
     spsdk_apps_common_options,
+    spsdk_output_option,
+    spsdk_sdp_interface,
+    spsdk_use_json_option,
 )
-from spsdk.apps.utils.utils import (
-    INT,
-    SPSDKAppError,
-    catch_spsdk_error,
-    format_raw_data,
-    get_interface,
-)
-from spsdk.sdp import SDP
+from spsdk.apps.utils.utils import INT, SPSDKAppError, catch_spsdk_error, format_raw_data
 from spsdk.sdp.commands import ResponseValue
+from spsdk.sdp.protocol.base import SDPProtocolBase
+from spsdk.sdp.sdp import SDP
+from spsdk.utils import misc
 
 
-@click.group(name="sdphost", no_args_is_help=True, cls=CommandsTreeGroup)
-@isp_interfaces(uart=True, usb=True, is_sdp=True, default_timeout=10000)
+@click.group(name="sdphost", cls=CommandsTreeGroup)
+@spsdk_sdp_interface()
+@spsdk_use_json_option
 @spsdk_apps_common_options
 @click.pass_context
 def main(
     ctx: click.Context,
-    port: str,
-    usb: str,
+    interface: SDPProtocolBase,
     use_json: bool,
     log_level: int,
-    timeout: int,
 ) -> int:
-    """Utility for communication with ROM on i.MX targets using SDP protocol (i.MX RT1xxx)."""
-    logging.basicConfig(level=log_level or logging.WARNING)
-    # if --help is provided anywhere on command line, skip interface lookup and display help message
-    if "--help" not in sys.argv[1:]:
-        ctx.obj = {
-            "interface": get_interface(module="sdp", port=port, usb=usb, timeout=timeout),
-            "use_json": use_json,
-        }
+    """Utility for communication with ROM on i.MX targets using SDP protocol."""
+    spsdk_logger.install(level=log_level)
+    ctx.obj = {
+        "interface": interface,
+        "use_json": use_json,
+    }
     return 0
 
 
-@main.command()
+@main.command(no_args_is_help=False)
 @click.pass_context
 def error_status(ctx: click.Context) -> None:
     """Reads the error code from the device."""
@@ -68,7 +63,7 @@ def error_status(ctx: click.Context) -> None:
     )
 
 
-@main.command()
+@main.command(no_args_is_help=True)
 @click.argument("address", type=INT(), required=True)
 @click.pass_context
 def jump_address(ctx: click.Context, address: int) -> None:
@@ -85,13 +80,13 @@ def jump_address(ctx: click.Context, address: int) -> None:
     display_output([], sdp.hab_status)
 
 
-@main.command()
+@main.command(no_args_is_help=True)
 @click.argument("address", type=INT(), required=True)
 @click.argument("bin_file", metavar="FILE", type=click.File("rb"), required=True)
 @click.argument("count", type=INT(), required=False)
 @click.pass_context
 def write_file(ctx: click.Context, address: int, bin_file: click.File, count: int) -> None:
-    """Writes file to the device’s memory address.
+    """Writes file to the device's memory address.
 
     \b
     ADDRESS - starting address of the image
@@ -108,11 +103,11 @@ def write_file(ctx: click.Context, address: int, bin_file: click.File, count: in
     )
 
 
-@main.command()
+@main.command(no_args_is_help=True)
 @click.argument("address", type=INT(), required=True)
-@click.argument("item_length", type=INT(), required=False, default=32, metavar="[FORMAT]")
+@click.argument("item_length", type=INT(), required=False, default="32", metavar="[FORMAT]")
 @click.argument("count", type=INT(), required=False, default=None)
-@click.argument("file", type=click.File("wb"), required=False)
+@spsdk_output_option(required=False)
 @click.option("-h", "--use-hexdump", is_flag=True, default=False, help="Use hexdump format")
 @click.pass_context
 def read_register(
@@ -120,7 +115,7 @@ def read_register(
     address: int,
     item_length: int,
     count: int,
-    file: click.File,
+    output: str,
     use_hexdump: bool,
 ) -> None:
     """Reads the contents of a memory location or register value.
@@ -133,7 +128,6 @@ def read_register(
     ADDRESS - starting address where to read
     FORMAT  - bits per item: valid values: 8, 16, 32; default 32
     COUNT   - bytes to read; default size of FORMAT
-    FILE    - write data into a file; write to stdout if not specified
     """
     with SDP(ctx.obj["interface"]) as sdp:
         response = sdp.read_safe(address, count, item_length)
@@ -141,15 +135,16 @@ def read_register(
         raise SPSDKAppError(
             f"Error: invalid sub-command or arguments 'read-register {address:#8X} {item_length} {count}'"
         )
-    if file:
-        file.write(response)  # type: ignore
+    if output:
+        misc.write_file(response, output, mode="wb")
+        click.echo(f"{len(response)} bytes written to {output}")
     else:
         click.echo(format_raw_data(response, use_hexdump=use_hexdump))
     display_output([], sdp.hab_status, ctx.obj["use_json"])
 
 
 @click.argument("baudrate", type=INT(), required=True)
-@main.command()
+@main.command(no_args_is_help=True)
 @click.pass_context
 def set_baudrate(ctx: click.Context, baudrate: int) -> None:
     """Configures UART baudrate.
@@ -207,7 +202,13 @@ def decode_status_code(status_code: Optional[int] = None) -> str:
     """
     if not status_code:
         return "UNKNOWN ERROR"
-    return f"{status_code} ({status_code:#x}) {ResponseValue.desc(status_code)}"
+    no_desc = "NO DESCRIPTION"
+    desc = (
+        ResponseValue.get_description(status_code, no_desc)
+        if ResponseValue.contains(status_code)
+        else no_desc
+    )
+    return f"{status_code} ({status_code:#x}) {desc}"
 
 
 @catch_spsdk_error
