@@ -1,53 +1,52 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2020-2023 NXP
+# Copyright 2020-2025 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
 """Console script for SDPS module aka SDPSHost."""
 
-import logging
 import sys
 
 import click
 
+from spsdk.apps.utils import spsdk_logger
 from spsdk.apps.utils.common_cli_options import (
     CommandsTreeGroup,
-    isp_interfaces,
+    is_click_help,
     spsdk_apps_common_options,
+    spsdk_family_option,
+    spsdk_sdp_interface,
 )
-from spsdk.apps.utils.utils import catch_spsdk_error, get_interface
-from spsdk.sdp import SDPS
-from spsdk.sdp.sdps import ROM_INFO
-
-WARNING_MSG = """
-!!! THIS IS AN EXPERIMENTAL UTILITY! USE WITH CAUTION !!!
-"""
+from spsdk.apps.utils.utils import WARNING_MSG, catch_spsdk_error
+from spsdk.sdp.protocol.base import SDPProtocolBase
+from spsdk.sdp.sdps import SDPS
+from spsdk.utils.family import FamilyRevision
 
 
-@click.group(name="sdpshost", no_args_is_help=True, cls=CommandsTreeGroup)
-@isp_interfaces(uart=True, usb=True, is_sdp=True, json_option=False)
-@click.option("-n", "--name", type=click.Choice(list(ROM_INFO.keys())), help="Name of the device")
+@click.group(name="sdpshost", cls=CommandsTreeGroup)
+@spsdk_sdp_interface(identify_by_family=True)
+@spsdk_family_option(families=SDPS.get_supported_families())
 @spsdk_apps_common_options
 @click.pass_context
-def main(ctx: click.Context, port: str, usb: str, name: str, log_level: int, timeout: int) -> int:
+def main(
+    ctx: click.Context, interface: SDPProtocolBase, family: FamilyRevision, log_level: int
+) -> int:
     """Utility for communication with ROM on i.MX targets using SDPS protocol (i.MX8/9)."""
-    logging.basicConfig(level=log_level or logging.WARNING)
+    spsdk_logger.install(level=log_level)
     click.echo(WARNING_MSG)
     # if --help is provided anywhere on command line, skip interface lookup and display help message
-    if "--help " in sys.argv:
-        port, usb = None, None  # type: ignore
+    if is_click_help(ctx, sys.argv):
+        return 0
     ctx.obj = {
-        "interface": get_interface(module="sdp", port=port, usb=usb, timeout=timeout)
-        if port or usb
-        else None,
-        "name": name,
+        "interface": interface,
+        "family": family,
     }
     return 0
 
 
-@main.command()
+@main.command(no_args_is_help=True)
 @click.argument("bin_file", metavar="FILE", type=click.File("rb"), required=True)
 @click.pass_context
 def write_file(ctx: click.Context, bin_file: click.File) -> None:
@@ -56,10 +55,10 @@ def write_file(ctx: click.Context, bin_file: click.File) -> None:
     \b
     FILE    - binary file to write
     """
-    click.echo(WARNING_MSG)
     data = bin_file.read()  # type: ignore
-    with SDPS(ctx.obj["interface"], device_name=ctx.obj["name"]) as sdps:
+    with SDPS(ctx.obj["interface"], family=ctx.obj["family"]) as sdps:
         sdps.write_file(data)
+    click.echo(f"Writing of file '{bin_file.name}' succeeded.")
 
 
 @catch_spsdk_error
