@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2020-2023 NXP
+# Copyright 2020-2025 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 """Module for NXP SPSDK DebugMailbox support."""
@@ -9,12 +9,13 @@
 import functools
 import logging
 from time import sleep
-from typing import Any, Dict, no_type_check
+from typing import Any, no_type_check
 
-from spsdk import SPSDKError
 from spsdk.debuggers.debug_probe import DebugProbe
-from spsdk.exceptions import SPSDKIOError
+from spsdk.exceptions import SPSDKError, SPSDKIOError
+from spsdk.utils.database import DatabaseManager
 from spsdk.utils.exceptions import SPSDKTimeoutError
+from spsdk.utils.family import FamilyRevision, get_db
 from spsdk.utils.misc import Timeout
 
 logger = logging.getLogger(__name__)
@@ -30,26 +31,38 @@ class DebugMailbox:
     def __init__(
         self,
         debug_probe: DebugProbe,
+        family: FamilyRevision,
         reset: bool = True,
-        moredelay: float = 1.0,
+        moredelay: float = 0.0,
         op_timeout: int = 1000,
     ) -> None:
         """Initialize DebugMailbox object.
 
         :param debug_probe: Debug probe instance.
+        :param family: Chip family.
         :param reset: Do reset of debug mailbox during initialization, defaults to True.
-        :param moredelay: Time of extra delay after reset sequence, defaults to 1.0.
-        :param op_timeout: Atomic operation timeout, defaults to 4000.
+        :param moredelay: Time of extra delay after reset sequence, defaults to 0.0.
+        :param op_timeout: Atomic operation timeout, defaults to 1000.
         :raises SPSDKIOError: Various kind of vulnerabilities during connection to debug mailbox.
         """
         # setup debug port / access point
-
-        self.debug_probe = debug_probe
+        self.family = family
         self.dbgmlbx_ap_ix = -1
+        self.non_standard_statuses = {}
+        self.command_delays: dict[str, float] = {}
+        if family:
+            db = get_db(family)
+            self.dbgmlbx_ap_ix = db.get_int(DatabaseManager.DAT, "dmbox_ap_ix", -1)
+            self.non_standard_statuses = db.get_dict(
+                DatabaseManager.DAT, "non_standard_statuses", {}
+            )
+            self.command_delays = db.get_dict(DatabaseManager.DAT, "command_delays", {})
+        self.debug_probe = debug_probe
+
         self.reset = reset
         self.moredelay = moredelay
         # setup registers and register bitfields
-        self.registers: Dict[str, Dict[str, Any]] = REGISTERS
+        self.registers: dict[str, dict[str, Any]] = REGISTERS
         # set internal operation timeout
         self.op_timeout = op_timeout
 
@@ -101,10 +114,11 @@ class DebugMailbox:
 
         :return: IDR value of debug mailbox AP.
         """
-        idr = self.dbgmlbx_reg_read(addr=REGISTERS["IDR"]["address"])
-        if idr != REGISTERS["IDR"]["expected"]:
+        idr = self.dbgmlbx_reg_read(addr=self.registers["IDR"]["address"])
+        if idr != self.registers["IDR"]["expected"]:
             logger.warning(
-                f"The read IDR value({hex(idr)}) doesn't match the expected value: {hex(REGISTERS['IDR']['expected'])}"
+                f"The read IDR value({hex(idr)}) doesn't match the expected "
+                f"value: {hex(self.registers['IDR']['expected'])}"
             )
         return idr
 
@@ -171,21 +185,24 @@ class DebugMailbox:
 
         :param func: Decorated function.
         """
-        POSSIBLE_DBGMLBX_AP_IX = [0, 2]
+        POSSIBLE_DBGMLBX_AP_IX = [2, 0, 1, 3, 8]
 
         @functools.wraps(func)
         def wrapper(self: "DebugMailbox", *args, **kwargs):
             if self.dbgmlbx_ap_ix < 0:
                 # Try to find DEBUG MAILBOX AP
+                logger.warning(
+                    "The debug mailbox access port index is not specified, trying autodetection."
+                )
                 for i in POSSIBLE_DBGMLBX_AP_IX:
                     try:
-                        idr = self.debug_probe.coresight_reg_read(
+                        idr = self.debug_probe.coresight_reg_read_safe(
                             access_port=True,
                             addr=self.debug_probe.get_coresight_ap_address(
-                                access_port=i, address=REGISTERS["IDR"]["address"]
+                                access_port=i, address=self.registers["IDR"]["address"]
                             ),
                         )
-                        if idr == REGISTERS["IDR"]["expected"]:
+                        if idr == self.registers["IDR"]["expected"]:
                             self.dbgmlbx_ap_ix = i
                             logger.debug(
                                 f"Found debug mailbox access port at AP{i}, IDR: 0x{idr:08X}"
@@ -211,7 +228,7 @@ class DebugMailbox:
         :return: The read value of addressed register (4 bytes)
         :raises NotImplementedError: Derived class has to implement this method
         """
-        return self.debug_probe.coresight_reg_read(
+        return self.debug_probe.coresight_reg_read_safe(
             addr=self.debug_probe.get_coresight_ap_address(
                 access_port=self.dbgmlbx_ap_ix, address=addr
             )
@@ -227,7 +244,7 @@ class DebugMailbox:
         :param data: the data to be written into register
         :raises NotImplementedError: Derived class has to implement this method
         """
-        self.debug_probe.coresight_reg_write(
+        self.debug_probe.coresight_reg_write_safe(
             addr=self.debug_probe.get_coresight_ap_address(
                 access_port=self.dbgmlbx_ap_ix, address=addr
             ),
@@ -235,7 +252,7 @@ class DebugMailbox:
         )
 
 
-REGISTERS: Dict[str, Any] = {
+REGISTERS: dict[str, Any] = {
     # Control and Status Word (CSW) is used to control
     # the Debug Mailbox communication
     "CSW": {
