@@ -2,26 +2,28 @@
 # -*- coding: UTF-8 -*-
 #
 # Copyright 2016-2018 Martin Olejar
-# Copyright 2019-2023 NXP
+# Copyright 2019-2025 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
 """Commands and responses used by MBOOT module."""
 
 from struct import pack, unpack, unpack_from
-from typing import Dict, List, Optional, Type
+from typing import Optional, Type
 
-from spsdk.utils.easy_enum import Enum
+from typing_extensions import Self
 
-from .error_codes import StatusCode
-from .exceptions import McuBootError
+from spsdk.mboot.error_codes import StatusCode
+from spsdk.mboot.exceptions import McuBootError
+from spsdk.utils.interfaces.commands import CmdPacketBase, CmdResponseBase
+from spsdk.utils.spsdk_enum import SpsdkEnum
 
 ########################################################################################################################
 # McuBoot Commands and Responses Tags
 ########################################################################################################################
 
 # fmt: off
-class CommandTag(Enum):
+class CommandTag(SpsdkEnum):
     """McuBoot Commands."""
 
     NO_COMMAND                  = (0x00, "NoCommand", "No Command")
@@ -49,6 +51,8 @@ class CommandTag(Enum):
     TRUST_PROVISIONING          = (0x16, "TrustProvisioning", "Trust Provisioning")
     FUSE_READ                   = (0x17, "ReadFuse", "Read Fuse")
     UPDATE_LIFE_CYCLE           = (0x18, "UpdateLifeCycle", "Update Life Cycle")
+    ELE_MESSAGE                 = (0x19, "EleMessage", "Send EdgeLock Enclave Message")
+    EL2GO                       = (0x20, "EL2GO", "EL2GO Provisioning Commands and API Calls")
 
     # reserved commands
     CONFIGURE_I2C = (0xC1, "ConfigureI2c", "Configure I2C")
@@ -57,7 +61,7 @@ class CommandTag(Enum):
 
 
 
-class CommandFlag(Enum):
+class CommandFlag(SpsdkEnum):
     """Flags for McuBoot commands."""
 
     NONE           = (0, "NoFlags", "No flags specified")
@@ -65,7 +69,7 @@ class CommandFlag(Enum):
 
 
 
-class ResponseTag(Enum):
+class ResponseTag(SpsdkEnum):
     """McuBoot Responses to Commands."""
 
     GENERIC                     = (0xA0, "GenericResponse", "Generic Response")
@@ -78,7 +82,7 @@ class ResponseTag(Enum):
     TRUST_PROVISIONING_RESPONSE = (0xB6, "TrustProvisioningResponse", "Trust Provisioning Response")
 
 
-class KeyProvOperation(Enum):
+class KeyProvOperation(SpsdkEnum):
     """Type of key provisioning operation."""
 
     ENROLL              = (0, "Enroll", "Enroll Operation")
@@ -90,21 +94,21 @@ class KeyProvOperation(Enum):
     READ_KEY_STORE      = (6, "ReadKeyStore", "Read Key Store Operation")
 
 
-class KeyProvUserKeyType(Enum):
+class KeyProvUserKeyType(SpsdkEnum):
     """Enumeration of supported user keys in PUF. Keys are SoC specific, not all will be supported for the processor."""
 
-    OTFADKEK        = (2, "OTFADKEK", "Key for OTFAD encryption")  # used on RTxxx
-    SBKEK           = (3, "SBKEK", "Key for SB file encryption")  # Available on LPC55Sxx and RTxxx
-    PRINCE_REGION_0 = (7, "PRINCE0", "Key for Prince region 0")  # LPC55Sxx
-    PRINCE_REGION_1 = (8, "PRINCE1", "Key for Prince region 1")  # LPC55Sxx
-    PRINCE_REGION_2 = (9, "PRINCE2", "Key for Prince region 2")  # LPC55Sxx
-    PRINCE_REGION_3 = (10, "PRINCE3", "Key for Prince region 3")  # NHS52xx
+    OTFADKEK        = (2, "OTFADKEK", "Key for OTFAD encryption")
+    SBKEK           = (3, "SBKEK", "Key for SB file encryption")
+    PRINCE_REGION_0 = (7, "PRINCE0", "Key for Prince region 0")
+    PRINCE_REGION_1 = (8, "PRINCE1", "Key for Prince region 1")
+    PRINCE_REGION_2 = (9, "PRINCE2", "Key for Prince region 2")
+    PRINCE_REGION_3 = (10, "PRINCE3", "Key for Prince region 3")
 
-    USERKEK         = (11, "USERKEK", "Encrypted boot image key")  # LPC55Sxx and RTxxx
-    UDS             = (12, "UDS", "Universal Device Secret for DICE")  # LPC55Sxx and RTxxx
+    USERKEK         = (11, "USERKEK", "Encrypted boot image key")
+    UDS             = (12, "UDS", "Universal Device Secret for DICE")
 
 
-class GenerateKeyBlobSelect(Enum):
+class GenerateKeyBlobSelect(SpsdkEnum):
     """Key selector for the generate-key-blob function.
 
     For devices with SNVS, valid options of [key_sel] are
@@ -119,32 +123,34 @@ class GenerateKeyBlobSelect(Enum):
     CMK     = (3, "CMK", "CMK from SNVS")
 
 
-class TrustProvOperation(Enum):
+class TrustProvOperation(SpsdkEnum):
     """Operations supported by Trust Provisioning flow."""
 
     PROVE_GENUINITY = (0xF4, "ProveGenuinity", "Start the proving genuinity process")
+    PROVE_GENUINITY_HYBRID = (0xF5, "ProveGenuinityHybrid", "Start the hybrid proving genuinity process")
     ISP_SET_WRAPPED_DATA = (0xF0, "SetWrappedData", "Start processing Wrapped data")
     """Type of trust provisioning operation."""
 
     OEM_GEN_MASTER_SHARE        = (0, "OemGenMasterShare", "Enroll Operation")
     OEM_SET_MASTER_SHARE        = (1, "SetUserKey", "Set User Key Operation")
-    OEM_GET_CUST_CERT_DICE_PUK  = (2, "SetIntrinsicKey", "Set Intrinsic Key Operation")
+    OEM_GET_CUST_CERT_DICE_PUK  = (2, "GetDiceCaPuk", "Get DICE CA public key")
     HSM_GEN_KEY                 = (3, "HsmGenKey", "HSM gen key")
     HSM_STORE_KEY               = (4, "HsmStoreKey", "HSM store key")
     HSM_ENC_BLOCK               = (5, "HsmEncBlock", "HSM Enc block")
     HSM_ENC_SIGN                = (6, "HsnEncSign", "HSM enc sign")
+    OEM_GET_CUST_DICE_RESPONSE  = (7, "GetDiceResponse", "Get DICE response")
 
-
-class TrustProvOemKeyType(Enum):
+class TrustProvOemKeyType(SpsdkEnum):
     """Type of oem key type definition."""
 
     MFWISK      = (0xC3A5, "MFWISK", "ECDSA Manufacturing Firmware Signing Key")
+    ENCKEY      = (0x3CA5, "ENCKEY", "Generic Encryption Key")
     MFWENCK     = (0xA5C3, "MFWENCK", "CKDF Master Key for Manufacturing Firmware Encryption Key")
     GENSIGNK    = (0x5A3C, "GENSIGNK", "Generic ECDSA Signing Key")
     GETCUSTMKSK = (0x3C5A, "GETCUSTMKSK", "CKDF Master Key for Production Firmware Encryption Key")
 
 
-class TrustProvKeyType(Enum):
+class TrustProvKeyType(SpsdkEnum):
     """Type of key type definition."""
 
     CKDFK = (1, "CKDFK", "CKDF Master Key")
@@ -155,12 +161,37 @@ class TrustProvKeyType(Enum):
     KUOK  = (6, "KUOK", "Key Unwrap Only Key")
 
 
-class TrustProvWrappingKeyType(Enum):
+class TrustProvWrappingKeyType(SpsdkEnum):
     """Type of wrapping key type definition."""
 
     INT_SK = (0x10, "INT_SK", "The wrapping key for wrapping of MFG_CUST_MK_SK0_BLOB")
     EXT_SK = (0x11, "EXT_SK", "The wrapping key for wrapping of MFG_CUST_MK_SK0_BLOB")
 
+
+class TrustProvWpc(SpsdkEnum):
+    """Type of WPC trusted facility commands for DSC."""
+
+    WPC_GET_ID              = (0x5000000, "wpc_get_id", "WPC get ID")
+    NXP_GET_ID              = (0x5000001, "nxp_get_id", "NXP get ID")
+    WPC_INSERT_CERT         = (0x5000002, "wpc_insert_cert", "WPC insert certificate")
+    WPC_SIGN_CSR            = (0x5000003, "wpc_sign_csr", "WPC sign CSR")
+
+
+class TrustProvDevHsmDsc(SpsdkEnum):
+    """Type of DSC Device HSM."""
+
+    DSC_HSM_CREATE_SESSION  = (0x6000000, "dsc_hsm_create_session", "DSC HSM create session")
+    DSC_HSM_ENC_BLK         = (0x6000001, "dsc_hsm_enc_blk", "DSC HSM encrypt bulk")
+    DSC_HSM_ENC_SIGN        = (0x6000002, "dsc_hsm_enc_sign", "DSC HSM sign")
+
+
+class EL2GOCommandGroup(SpsdkEnum):
+    """EL2GO command group."""
+
+    EL2GO_GET_FW_VERSION    = (0x1, "el2go_get_version", "EL2GO Get Version")
+    EL2GO_CLOSE_DEVICE      = (0x2, "el2go_close_device", "EL2GO Close Device")
+    EL2GO_BATCH_TP          = (0x3, "el2go_batch_tp" , "EL2GO Batch Trust Provisioning")
+
 # fmt: on
 
 ########################################################################################################################
@@ -192,26 +223,29 @@ class CmdHeader:
     def __ne__(self, obj: object) -> bool:
         return not self.__eq__(obj)
 
-    def __str__(self) -> str:
+    def __repr__(self) -> str:
         return f"<Tag=0x{self.tag:02X}, Flags=0x{self.flags:02X}, ParamsCount={self.params_count}>"
 
-    def __repr__(self) -> str:
+    def __str__(self) -> str:
         return (
             f"CmdHeader(tag=0x{self.tag:02X}, flags=0x{self.flags:02X}, "
             f"reserved={self.reserved}, params_count={self.params_count})"
         )
 
-    def to_bytes(self) -> bytes:
-        """Serialize header into bytes."""
+    def export(self) -> bytes:
+        """Export the command header to bytes.
+
+        :return: Exported command header as bytes
+        """
         return pack("4B", self.tag, self.flags, self.reserved, self.params_count)
 
     @classmethod
-    def from_bytes(cls, data: bytes, offset: int = 0) -> "CmdHeader":
-        """Deserialize header from bytes.
+    def parse(cls, data: bytes, offset: int = 0) -> Self:
+        """Parse header from bytes.
 
         :param data: Input data in bytes
         :param offset: The offset of input data
-        :return: De-serialized CmdHeader object
+        :return: Parsed CmdHeader object
         :raises McuBootError: Invalid data format
         """
         if len(data) < 4:
@@ -219,13 +253,15 @@ class CmdHeader:
         return cls(*unpack_from("4B", data, offset))
 
 
-class CmdPacket:
+class CmdPacket(CmdPacketBase):
     """McuBoot command packet format class."""
 
     SIZE = 32
     EMPTY_VALUE = 0x00
 
-    def __init__(self, tag: int, flags: int, *args: int, data: Optional[bytes] = None) -> None:
+    def __init__(
+        self, tag: CommandTag, flags: int, *args: int, data: Optional[bytes] = None
+    ) -> None:
         """Initialize the Command Packet object.
 
         :param tag: Tag identifying the command
@@ -233,7 +269,7 @@ class CmdPacket:
         :param args: Arguments used by the command
         :param data: Additional data, defaults to None
         """
-        self.header = CmdHeader(tag, flags, 0, len(args))
+        self.header = CmdHeader(tag.tag, flags, 0, len(args))
         self.params = list(args)
         if data is not None:
             if len(data) % 4:
@@ -248,30 +284,31 @@ class CmdPacket:
         return not self.__eq__(obj)
 
     def __str__(self) -> str:
-        return "<" + self.info() + ">"
-
-    def info(self) -> str:
         """Get object info."""
-        tag = CommandTag.get(self.header.tag, f"0x{self.header.tag:02X}")
+        tag = (
+            CommandTag.get_label(self.header.tag)
+            if self.header.tag in CommandTag.tags()
+            else f"0x{self.header.tag:02X}"
+        )
         return f"Tag={tag}, Flags=0x{self.header.flags:02X}" + "".join(
             f", P[{n}]=0x{param:08X}" for n, param in enumerate(self.params)
         )
 
-    def to_bytes(self, padding: bool = True) -> bytes:
-        """Serialize CmdPacket into bytes.
+    def export(self, padding: bool = True) -> bytes:
+        """Export CmdPacket into bytes.
 
         :param padding: If True, add padding to specific size
-        :return: Serialized object into bytes
+        :return: Exported object into bytes
         """
         self.header.params_count = len(self.params)
-        data = self.header.to_bytes()
+        data = self.header.export()
         data += pack(f"<{self.header.params_count}I", *self.params)
         if padding and len(data) < self.SIZE:
             data += bytes([self.EMPTY_VALUE] * (self.SIZE - len(data)))
         return data
 
 
-class CmdResponse:
+class CmdResponse(CmdResponseBase):
     """McuBoot response base format class."""
 
     def __init__(self, header: CmdHeader, raw_data: bytes) -> None:
@@ -287,6 +324,18 @@ class CmdResponse:
         (status,) = unpack_from("<L", raw_data)
         self.status: int = status
 
+    @property
+    def value(self) -> int:
+        """Return a integer representation of the response."""
+        return unpack_from(">I", self.raw_data)[0]
+
+    def _get_status_label(self) -> str:
+        return (
+            StatusCode.get_label(self.status)
+            if self.status in StatusCode.tags()
+            else f"Unknown[0x{self.status:08X}]"
+        )
+
     def __eq__(self, obj: object) -> bool:
         return isinstance(obj, CmdResponse) and vars(obj) == vars(self)
 
@@ -294,9 +343,6 @@ class CmdResponse:
         return not self.__eq__(obj)
 
     def __str__(self) -> str:
-        return "<" + self.info() + ">"
-
-    def info(self) -> str:
         """Get object info."""
         return (
             f"Tag=0x{self.header.tag:02X}, Flags=0x{self.header.flags:02X}"
@@ -319,11 +365,15 @@ class GenericResponse(CmdResponse):
         _, tag = unpack_from("<2I", raw_data)
         self.cmd_tag: int = tag
 
-    def info(self) -> str:
+    def __str__(self) -> str:
         """Get object info."""
-        tag = ResponseTag.name(self.header.tag)
-        status = StatusCode.get(self.status, f"Unknown[0x{self.status:08X}]")
-        cmd = CommandTag.get(self.cmd_tag, f"Unknown[0x{self.cmd_tag:02X}]")
+        tag = ResponseTag.get_label(self.header.tag)
+        status = self._get_status_label()
+        cmd = (
+            CommandTag.get_label(self.cmd_tag)
+            if self.cmd_tag in CommandTag.tags()
+            else f"Unknown[0x{self.cmd_tag:02X}]"
+        )
         return f"Tag={tag}, Status={status}, Cmd={cmd}"
 
 
@@ -338,12 +388,12 @@ class GetPropertyResponse(CmdResponse):
         """
         super().__init__(header, raw_data)
         _, *values = unpack_from(f"<{self.header.params_count}I", raw_data)
-        self.values: List[int] = list(values)
+        self.values: list[int] = list(values)
 
-    def info(self) -> str:
+    def __str__(self) -> str:
         """Get object info."""
-        tag = ResponseTag.name(self.header.tag)
-        status = StatusCode.get(self.status, f"Unknown[0x{self.status:08X}]")
+        tag = ResponseTag.get_label(self.header.tag)
+        status = self._get_status_label()
         return f"Tag={tag}, Status={status}" + "".join(
             f", v{n}=0x{value:08X}" for n, value in enumerate(self.values)
         )
@@ -362,10 +412,10 @@ class ReadMemoryResponse(CmdResponse):
         _, length = unpack_from("<2I", raw_data)
         self.length: int = length
 
-    def info(self) -> str:
+    def __str__(self) -> str:
         """Get object info."""
-        tag = ResponseTag.name(self.header.tag)
-        status = StatusCode.get(self.status, f"Unknown[0x{self.status:08X}]")
+        tag = ResponseTag.get_label(self.header.tag)
+        status = self._get_status_label()
         return f"Tag={tag}, Status={status}, Length={self.length}"
 
 
@@ -381,13 +431,13 @@ class FlashReadOnceResponse(CmdResponse):
         super().__init__(header, raw_data)
         _, length, *values = unpack_from(f"<{self.header.params_count}I", raw_data)
         self.length: int = length
-        self.values: List[int] = list(values)
+        self.values: list[int] = list(values)
         self.data = raw_data[8 : 8 + self.length] if self.length > 0 else b""
 
-    def info(self) -> str:
+    def __str__(self) -> str:
         """Get object info."""
-        tag = ResponseTag.name(self.header.tag)
-        status = StatusCode.get(self.status, f"Unknown[0x{self.status:08X}]")
+        tag = ResponseTag.get_label(self.header.tag)
+        status = self._get_status_label()
         return f"Tag={tag}, Status={status}, Length={self.length}"
 
 
@@ -404,10 +454,10 @@ class FlashReadResourceResponse(CmdResponse):
         _, length = unpack_from("<2I", raw_data)
         self.length: int = length
 
-    def info(self) -> str:
+    def __str__(self) -> str:
         """Get object info."""
-        tag = ResponseTag.name(self.header.tag)
-        status = StatusCode.get(self.status, f"Unknown[0x{self.status:08X}]")
+        tag = ResponseTag.get_label(self.header.tag)
+        status = self._get_status_label()
         return f"Tag={tag}, Status={status}, Length={self.length}"
 
 
@@ -424,10 +474,10 @@ class KeyProvisioningResponse(CmdResponse):
         _, length = unpack_from("<2I", raw_data)
         self.length: int = length
 
-    def info(self) -> str:
+    def __str__(self) -> str:
         """Get object info."""
-        tag = ResponseTag.name(self.header.tag)
-        status = StatusCode.get(self.status, f"Unknown[0x{self.status:08X}]")
+        tag = ResponseTag.get_label(self.header.tag)
+        status = self._get_status_label()
         return f"Tag={tag}, Status={status}, Length={self.length}"
 
 
@@ -442,14 +492,24 @@ class TrustProvisioningResponse(CmdResponse):
         """
         super().__init__(header, raw_data)
         _, *values = unpack(f"<{self.header.params_count}I", raw_data)
-        self.values: List[int] = list(values)
+        self.values: list[int] = list(values)
 
-    def info(self) -> str:
+    def __str__(self) -> str:
         """Get object info."""
-        tag = ResponseTag.name(self.header.tag)
-        status = StatusCode.get(self.status, f"Unknown[0x{self.status:08X}]")
+        tag = ResponseTag.get_label(self.header.tag)
+        status = self._get_status_label()
         return f"Tag={tag}, Status={status}"
 
+    def get_payload_data(self, offset: int = 0) -> Optional[bytes]:
+        """Get the payload from the response.
+
+        :param offset: Offset of integer unit in data
+        """
+        if len(self.values) < 2:
+            return None
+        blocks = [x.to_bytes(length=4, byteorder="little") for x in self.values[offset:]]
+        return b"".join(blocks)
+
 
 class NoResponse(CmdResponse):
     """Special internal case when no response is provided by the target."""
@@ -460,7 +520,7 @@ class NoResponse(CmdResponse):
         :param cmd_tag: Tag of the command that preceded the no-response from target
         """
         header = CmdHeader(tag=cmd_tag, flags=0, reserved=0, params_count=0)
-        raw_data = pack("<L", StatusCode.NO_RESPONSE)
+        raw_data = pack("<L", StatusCode.NO_RESPONSE.tag)
         super().__init__(header, raw_data)
 
 
@@ -469,19 +529,19 @@ def parse_cmd_response(data: bytes, offset: int = 0) -> CmdResponse:
 
     :param data: Input data in bytes
     :param offset: The offset of input data
-    :return: De-serialized object from data
+    :return: Parsed command response from data
     """
-    known_response: Dict[int, Type[CmdResponse]] = {
-        ResponseTag.GENERIC: GenericResponse,
-        ResponseTag.GET_PROPERTY: GetPropertyResponse,
-        ResponseTag.READ_MEMORY: ReadMemoryResponse,
-        ResponseTag.FLASH_READ_RESOURCE: FlashReadResourceResponse,
-        ResponseTag.FLASH_READ_ONCE: FlashReadOnceResponse,
-        ResponseTag.KEY_BLOB_RESPONSE: ReadMemoryResponse,  # not sure what format is returned, this work on RT1050
-        ResponseTag.KEY_PROVISIONING_RESPONSE: KeyProvisioningResponse,
-        ResponseTag.TRUST_PROVISIONING_RESPONSE: TrustProvisioningResponse,
+    known_response: dict[int, Type[CmdResponse]] = {
+        ResponseTag.GENERIC.tag: GenericResponse,
+        ResponseTag.GET_PROPERTY.tag: GetPropertyResponse,
+        ResponseTag.READ_MEMORY.tag: ReadMemoryResponse,
+        ResponseTag.FLASH_READ_RESOURCE.tag: FlashReadResourceResponse,
+        ResponseTag.FLASH_READ_ONCE.tag: FlashReadOnceResponse,
+        ResponseTag.KEY_BLOB_RESPONSE.tag: ReadMemoryResponse,
+        ResponseTag.KEY_PROVISIONING_RESPONSE.tag: KeyProvisioningResponse,
+        ResponseTag.TRUST_PROVISIONING_RESPONSE.tag: TrustProvisioningResponse,
     }
-    header = CmdHeader.from_bytes(data, offset)
+    header = CmdHeader.parse(data, offset)
     if header.tag in known_response:
         return known_response[header.tag](header, data[CmdHeader.SIZE :])
 
