@@ -2,23 +2,31 @@
 # -*- coding: UTF-8 -*-
 #
 # Copyright 2016-2018 Martin Olejar
-# Copyright 2019-2023 NXP
+# Copyright 2019-2025 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
 """Helper module for more human-friendly interpretation of the target device properties."""
 
-
 import ctypes
+import logging
 from copy import deepcopy
-from typing import List, Optional, Tuple, Type, Union
+from enum import Enum
+from typing import Callable, Optional, Type, Union
 
+from typing_extensions import Self
+
+from spsdk.exceptions import SPSDKError, SPSDKKeyError, SPSDKValueError
+from spsdk.mboot.commands import CommandTag
+from spsdk.mboot.error_codes import StatusCode
 from spsdk.mboot.exceptions import McuBootError
-from spsdk.utils.easy_enum import Enum
+from spsdk.mboot.memories import ExtMemPropTags, MemoryRegion
+from spsdk.utils.database import DatabaseManager
+from spsdk.utils.family import FamilyRevision, get_db
+from spsdk.utils.misc import Endianness
+from spsdk.utils.spsdk_enum import SpsdkEnum
 
-from .commands import CommandTag
-from .error_codes import StatusCode
-from .memories import ExtMemPropTags, MemoryRegion
+logger = logging.getLogger(__name__)
 
 
 ########################################################################################################################
@@ -41,6 +49,21 @@ def size_fmt(value: Union[int, float], kibibyte: bool = True) -> str:
     return f"{value} {x}" if x == "B" else f"{value:3.1f} {x}"
 
 
+def int_fmt(value: int, format_str: str) -> str:
+    """Get stringified integer representation."""
+    if format_str == "size":
+        str_value = size_fmt(value)
+    elif format_str == "hex":
+        str_value = f"0x{value:08X}"
+    elif format_str == "dec":
+        str_value = str(value)
+    elif format_str == "int32":
+        str_value = str(ctypes.c_int32(value).value)
+    else:
+        str_value = format_str.format(value)
+    return str_value
+
+
 ########################################################################################################################
 # McuBoot helper classes
 ########################################################################################################################
@@ -84,12 +107,12 @@ class Version:
     def __ge__(self, obj: "Version") -> bool:
         return self.to_int(True) >= obj.to_int(True)
 
-    def __str__(self) -> str:
-        return self.to_str()
-
     def __repr__(self) -> str:
         return f"<Version(mark={self.mark}, major={self.major}, minor={self.minor}, fixation={self.fixation})>"
 
+    def __str__(self) -> str:
+        return self.to_str()
+
     def from_int(self, value: int) -> None:
         """Parse version data from raw int value.
 
@@ -143,50 +166,154 @@ class Version:
 # fmt: off
 class PropertyTag(Enum):
     """McuBoot Properties."""
+    LIST_PROPERTIES            = ('ListProperties', 'list-properties', 'List Properties')
+    CURRENT_VERSION            = ('CurrentVersion', 'current-version', 'Current Version')
+    AVAILABLE_PERIPHERALS      = ('AvailablePeripherals', 'available-peripherals', 'Available Peripherals')
+    FLASH_START_ADDRESS        = ('FlashStartAddress', 'flash-start-address', 'Flash Start Address')
+    FLASH_SIZE                 = ('FlashSize', 'flash-size-in-bytes', 'Flash Size')
+    FLASH_SECTOR_SIZE          = ('FlashSectorSize', 'flash-sector-size', 'Flash Sector Size')
+    FLASH_BLOCK_COUNT          = ('FlashBlockCount', 'flash-block-count', 'Flash Block Count')
+    AVAILABLE_COMMANDS         = ('AvailableCommands', 'available-commands', 'Available Commands')
+    CRC_CHECK_STATUS           = ('CrcCheckStatus', 'check-status', 'CRC Check Status')
+    LAST_ERROR                 = ('LastError', 'reserved', 'Last Error Value')
+    VERIFY_WRITES              = ('VerifyWrites', 'verify-writes', 'Verify Writes')
+    VERIFY_ERASE               = ('VerifyErase', 'verify-erase', 'Verify Erase')
+    MAX_PACKET_SIZE            = ('MaxPacketSize', 'max-packet-size', 'Max Packet Size')
+    RESERVED_REGIONS           = ('ReservedRegions', 'reserved-regions', 'Reserved Regions')
+    VALIDATE_REGIONS           = ('ValidateRegions', 'reserved_1', 'Validate Regions')
+    RAM_START_ADDRESS          = ('RamStartAddress', 'ram-start-address', 'RAM Start Address')
+    RAM_SIZE                   = ('RamSize', 'ram-size-in-bytes', 'RAM Size')
+    SYSTEM_DEVICE_IDENT        = ('SystemDeviceIdent', 'system-device-id', 'System Device Identification')
+    FLASH_SECURITY_STATE       = ('FlashSecurityState', 'security-state', 'Security State')
+    LIFE_CYCLE_STATE           = ('LifeCycleState', 'life-cycle', 'Life Cycle State')
+    UNIQUE_DEVICE_IDENT        = ('UniqueDeviceIdent', 'unique-device-id', 'Unique Device Identification')
+    FLASH_FAC_SUPPORT          = ('FlashFacSupport', 'flash-fac-support', 'Flash Fac. Support')
+    FLASH_ACCESS_SEGMENT_SIZE  = ('FlashAccessSegmentSize', 'flash-access-segment-size', 'Flash Access Segment Size')
+    FLASH_ACCESS_SEGMENT_COUNT = ('FlashAccessSegmentCount', 'flash-access-segment-count', 'Flash Access Segment Count')
+    FLASH_READ_MARGIN          = ('FlashReadMargin', 'flash-read-margin', 'Flash Read Margin')
+    QSPI_INIT_STATUS           = ('QspiInitStatus', 'qspi/otfad-init-status', 'QuadSPI Initialization Status')
+    TARGET_VERSION             = ('TargetVersion', 'target-version', 'Target Version')
+    EXTERNAL_MEMORY_ATTRIBUTES = ('ExternalMemoryAttributes', 'external-memory-attributes', 'External Memory Attributes') # pylint: disable=line-too-long
+    RELIABLE_UPDATE_STATUS     = ('ReliableUpdateStatus', 'reliable-update-status', 'Reliable Update Status')
+    FLASH_PAGE_SIZE            = ('FlashPageSize', 'flash-page-size', 'Flash Page Size')
+    IRQ_NOTIFIER_PIN           = ('IrqNotifierPin', 'irq-notify-pin', 'Irq Notifier Pin')
+    PFR_KEYSTORE_UPDATE_OPT    = ('PfrKeystoreUpdateOpt', 'pfr-keystore-update-opt', 'PFR Keystore Update Opt')
+    BYTE_WRITE_TIMEOUT_MS      = ('ByteWriteTimeoutMs', 'byte-write-timeout-ms', 'Byte Write Timeout in ms')
+    FUSE_LOCKED_STATUS         = ('FuseLockedStatus', 'fuse-locked-status', 'Fuse Locked Status')
+    BOOT_STATUS_REGISTER       = ('BootStatusRegister', 'boot-status', 'Boot Status Register')
+    FIRMWARE_VERSION           = ('FirmwareVersion', 'loadable-fw-version', 'Firmware Version')
+    FUSE_PROGRAM_VOLTAGE       = ('FuseProgramVoltage', 'fuse-program-voltage', 'Fuse Program Voltage')
+    SHE_FLASH_PARTITION        = ('SheFlashPartition', 'she-flash-partition', 'Secure Hardware Extension: Flash Partition') # pylint: disable=line-too-long
+    SHE_BOOT_MODE              = ('SheBootMode', 'she-boot-mode', 'Secure Hardware Extension: Boot Mode')
+    UNKNOWN                    = ('Unknown', 'unknown', 'Unknown property')
 
-    # LIST_PROPERTIES            = (0x00, 'ListProperties', 'List Properties')
-    CURRENT_VERSION            = (0x01, "CurrentVersion", "Current Version")
-    AVAILABLE_PERIPHERALS      = (0x02, "AvailablePeripherals", "Available Peripherals")
-    FLASH_START_ADDRESS        = (0x03, "FlashStartAddress", "Flash Start Address")
-    FLASH_SIZE                 = (0x04, "FlashSize", "Flash Size")
-    FLASH_SECTOR_SIZE          = (0x05, "FlashSectorSize", "Flash Sector Size")
-    FLASH_BLOCK_COUNT          = (0x06, "FlashBlockCount", "Flash Block Count")
-    AVAILABLE_COMMANDS         = (0x07, "AvailableCommands", "Available Commands")
-    CRC_CHECK_STATUS           = (0x08, "CrcCheckStatus", "CRC Check Status")
-    LAST_ERROR                 = (0x09, "LastError", "Last Error Value")
-    VERIFY_WRITES              = (0x0A, "VerifyWrites", "Verify Writes")
-    MAX_PACKET_SIZE            = (0x0B, "MaxPacketSize", "Max Packet Size")
-    RESERVED_REGIONS           = (0x0C, "ReservedRegions", "Reserved Regions")
-    VALIDATE_REGIONS           = (0x0D, "ValidateRegions", "Validate Regions")
-    RAM_START_ADDRESS          = (0x0E, "RamStartAddress", "RAM Start Address")
-    RAM_SIZE                   = (0x0F, "RamSize", "RAM Size")
-    SYSTEM_DEVICE_IDENT        = (0x10, "SystemDeviceIdent", "System Device Identification")
-    FLASH_SECURITY_STATE       = (0x11, "FlashSecurityState", "Security State")
-    UNIQUE_DEVICE_IDENT        = (0x12, "UniqueDeviceIdent", "Unique Device Identification")
-    FLASH_FAC_SUPPORT          = (0x13, "FlashFacSupport", "Flash Fac. Support")
-    FLASH_ACCESS_SEGMENT_SIZE  = (0x14, "FlashAccessSegmentSize", "Flash Access Segment Size",)
-    FLASH_ACCESS_SEGMENT_COUNT = (0x15, "FlashAccessSegmentCount", "Flash Access Segment Count",)
-    FLASH_READ_MARGIN          = (0x16, "FlashReadMargin", "Flash Read Margin")
-    QSPI_INIT_STATUS           = (0x17, "QspiInitStatus", "QuadSPI Initialization Status")
-    TARGET_VERSION             = (0x18, "TargetVersion", "Target Version")
-    EXTERNAL_MEMORY_ATTRIBUTES = (0x19, "ExternalMemoryAttributes", "External Memory Attributes",)
-    RELIABLE_UPDATE_STATUS     = (0x1A, "ReliableUpdateStatus", "Reliable Update Status")
-    FLASH_PAGE_SIZE            = (0x1B, "FlashPageSize", "Flash Page Size")
-    IRQ_NOTIFIER_PIN           = (0x1C, "IrqNotifierPin", "Irq Notifier Pin")
-    PFR_KEYSTORE_UPDATE_OPT    = (0x1D, "PfrKeystoreUpdateOpt", "PFR Keystore Update Opt")
-    BYTE_WRITE_TIMEOUT_MS      = (0x1E, "ByteWriteTimeoutMs", "Byte Write Timeout in ms")
-    FUSE_LOCKED_STATUS         = (0x1F, "FuseLockedStatus", "Fuse Locked Status")
+    @property
+    def label(self) -> str:
+        """The internal name of the property tag."""
+        return self.value[0]
 
+    @property
+    def friendly_name(self) -> str:
+        """The friendly name of the property tag."""
+        return self.value[1]
 
-class PropertyTagKw45xx(Enum):
-    """McuBoot Properties."""
+    @property
+    def description(self) -> str:
+        """The description of the property tag."""
+        return self.value[2]
 
-    VERIFY_ERASE               = (0x0A, "VerifyErase", "Verify Erase")
-    BOOT_STATUS_REGISTER       = (0x14, "BootStatusRegister", "Boot Status Register",)
-    FIRMWARE_VERSION           = (0x15, "FirmwareVersion", "Firmware Version",)
-    FUSE_PROGRAM_VOLTAGE       = (0x16, "FuseProgramVoltage", "Fuse Program Voltage")
+    @classmethod
+    def from_name(cls, name: str) -> Self:
+        """Convert a name to its corresponding PropertyTag.
 
-class PeripheryTag(Enum):
+        :param name: The name to convert
+        :return: The matching PropertyTag
+        :raises SPSDKValueError: If no matching PropertyTag is found
+        """
+        for item in cls.__members__.values():
+            if item.label == name or item.friendly_name == name:
+                return item
+        raise SPSDKValueError(f"There is no {cls.__name__} item with name {name} defined")
+
+    @classmethod
+    def from_index(cls, index: int, family: Optional[FamilyRevision] = None) -> "PropertyTag":
+        """Convert a name to its corresponding PropertyTag.
+
+        :param index: Property index
+        :param family: Device family
+        :return: The matching PropertyTag
+        :raises SPSDKError: If no matching PropertyTag is found
+        """
+        properties = get_properties(family)
+        for idx, prop in properties.items():
+            if idx == index:
+                return prop
+        raise SPSDKError(f"No such a property with index {index} found")
+COMMON_PROPERTY_INDEXES = {
+    0x00: PropertyTag.LIST_PROPERTIES,
+    0x01: PropertyTag.CURRENT_VERSION,
+    0x02: PropertyTag.AVAILABLE_PERIPHERALS,
+    0x03: PropertyTag.FLASH_START_ADDRESS,
+    0x04: PropertyTag.FLASH_SIZE,
+    0x05: PropertyTag.FLASH_SECTOR_SIZE,
+    0x06: PropertyTag.FLASH_BLOCK_COUNT,
+    0x07: PropertyTag.AVAILABLE_COMMANDS,
+    0x08: PropertyTag.CRC_CHECK_STATUS,
+    0x09: PropertyTag.LAST_ERROR,
+    0x0A: PropertyTag.VERIFY_WRITES,
+    0x0B: PropertyTag.MAX_PACKET_SIZE,
+    0x0C: PropertyTag.RESERVED_REGIONS,
+    0x0D: PropertyTag.VALIDATE_REGIONS,
+    0x0E: PropertyTag.RAM_START_ADDRESS,
+    0x0F: PropertyTag.RAM_SIZE,
+    0x10: PropertyTag.SYSTEM_DEVICE_IDENT,
+    0x11: PropertyTag.FLASH_SECURITY_STATE,
+    0x12: PropertyTag.UNIQUE_DEVICE_IDENT,
+    0x13: PropertyTag.FLASH_FAC_SUPPORT,
+    0x14: PropertyTag.FLASH_ACCESS_SEGMENT_SIZE,
+    0x15: PropertyTag.FLASH_ACCESS_SEGMENT_COUNT,
+    0x16: PropertyTag.FLASH_READ_MARGIN,
+    0x17: PropertyTag.QSPI_INIT_STATUS,
+    0x18: PropertyTag.TARGET_VERSION,
+    0x19: PropertyTag.EXTERNAL_MEMORY_ATTRIBUTES,
+    0x1A: PropertyTag.RELIABLE_UPDATE_STATUS,
+    0x1B: PropertyTag.FLASH_PAGE_SIZE,
+    0x1C: PropertyTag.IRQ_NOTIFIER_PIN,
+    0x1D: PropertyTag.PFR_KEYSTORE_UPDATE_OPT,
+    0x1E: PropertyTag.BYTE_WRITE_TIMEOUT_MS,
+    0x1F: PropertyTag.FUSE_LOCKED_STATUS,
+    0x20: PropertyTag.BOOT_STATUS_REGISTER,
+    0x21: PropertyTag.FIRMWARE_VERSION,
+    0x22: PropertyTag.FUSE_PROGRAM_VOLTAGE,
+    0x24: PropertyTag.SHE_FLASH_PARTITION,
+    0x25: PropertyTag.SHE_BOOT_MODE,
+    0xFF: PropertyTag.UNKNOWN,
+}
+
+def get_property_index(prop:Union[PropertyTag, int], family:Optional[FamilyRevision] = None) -> int:
+    """Get index of given property."""
+    if isinstance(prop, int):
+        return prop
+    properties = get_properties(family)
+    for idx, prop_obj in properties.items():
+        if prop == prop_obj:
+            return idx
+    raise SPSDKError(f"Unknown property: {prop.name}")
+
+def get_properties(family: Optional[FamilyRevision] = None)-> dict[int, PropertyTag]:
+    """Get all properties including family specific properties if family defined."""
+    property_indexes = deepcopy(COMMON_PROPERTY_INDEXES)
+    if family:
+        try:
+            overridden_properties = get_db(family).get_dict(DatabaseManager().BLHOST, "overridden_properties", {})
+            for index, name in overridden_properties.items():
+                property_indexes[index] = PropertyTag.from_name(name)
+        except SPSDKValueError:
+            logger.warning(f"Family '{family}' has no blhost support defined in the database."
+                           "Please verify the family name is correct.")
+    return property_indexes
+
+class PeripheryTag(SpsdkEnum):
     """Tags representing peripherals."""
 
     UART      = (0x01, "UART", "UART Interface")
@@ -199,19 +326,19 @@ class PeripheryTag(Enum):
     LIN       = (0x80, "LIN", "LIN Interface")
 
 
-class FlashReadMargin(Enum):
+class FlashReadMargin(SpsdkEnum):
     """Scopes for flash read."""
 
-    NORMAL  = (0, "Normal")
-    USER    = (1, "User")
-    FACTORY = (2, "Factory")
+    NORMAL  = (0, "NORMAL")
+    USER    = (1, "USER")
+    FACTORY = (2, "FACTORY")
 
 
-class PfrKeystoreUpdateOpt(Enum):
+class PfrKeystoreUpdateOpt(SpsdkEnum):
     """Options for PFR updating."""
 
-    KEY_PROVISIONING = (0, "KeyProvisioning")
-    WRITE_MEMORY     = (1, "WriteMemory")
+    KEY_PROVISIONING = (0, "KEY_PROVISIONING", "KeyProvisioning")
+    WRITE_MEMORY     = (1, "WRITE_MEMORY", "WriteMemory")
 # fmt: on
 
 ########################################################################################################################
@@ -222,18 +349,25 @@ class PfrKeystoreUpdateOpt(Enum):
 class PropertyValueBase:
     """Base class for property value."""
 
-    __slots__ = ("tag", "name", "desc")
+    __slots__ = ("prop", "name", "desc")
 
-    def __init__(self, tag: int, name: Optional[str] = None, desc: Optional[str] = None) -> None:
+    def __init__(
+        self, prop: PropertyTag, name: Optional[str] = None, desc: Optional[str] = None
+    ) -> None:
         """Initialize the base of property.
 
         :param tag: Property tag, see: `PropertyTag`
         :param name: Optional name for the property
         :param desc: Optional description for the property
         """
-        self.tag = tag
-        self.name = name or PropertyTag.get(tag, "")
-        self.desc = desc or PropertyTag.desc(tag)
+        self.prop = prop
+        self.name = name or prop.label
+        self.desc = desc or prop.description
+
+    @property
+    def tag(self) -> int:
+        """Property index."""
+        return get_property_index(self.prop)
 
     def __str__(self) -> str:
         return f"{self.desc} = {self.to_str()}"
@@ -257,14 +391,14 @@ class IntValue(PropertyValueBase):
         "_fmt",
     )
 
-    def __init__(self, tag: int, raw_values: List[int], str_format: str = "dec") -> None:
+    def __init__(self, prop: PropertyTag, raw_values: list[int], str_format: str = "dec") -> None:
         """Initialize the integer-based property object.
 
         :param tag: Property tag, see: `PropertyTag`
         :param raw_values: List of integers representing the property
         :param str_format: Format to display the value ('dec', 'hex', 'size')
         """
-        super().__init__(tag)
+        super().__init__(prop)
         self._fmt = str_format
         self.value = raw_values[0]
 
@@ -274,17 +408,37 @@ class IntValue(PropertyValueBase):
 
     def to_str(self) -> str:
         """Get stringified property representation."""
-        if self._fmt == "size":
-            str_value = size_fmt(self.value)
-        elif self._fmt == "hex":
-            str_value = f"0x{self.value:08X}"
-        elif self._fmt == "dec":
-            str_value = str(self.value)
-        elif self._fmt == "int32":
-            str_value = str(ctypes.c_int32(self.value).value)
-        else:
-            str_value = self._fmt.format(self.value)
-        return str_value
+        return int_fmt(self.value, self._fmt)
+
+
+class IntListValue(PropertyValueBase):
+    """List of integers property."""
+
+    __slots__ = ("value", "_fmt", "delimiter")
+
+    def __init__(
+        self,
+        prop: PropertyTag,
+        raw_values: list[int],
+        str_format: str = "hex",
+        delimiter: str = ", ",
+    ) -> None:
+        """Initialize the integer-list-based property object.
+
+        :param tag: Property tag, see: `PropertyTag`
+        :param raw_values: List of integers representing the property
+        :param str_format: Format to display the value ('dec', 'hex', 'size')
+        :param delimiter: Delimiter for values in a list
+        """
+        super().__init__(prop)
+        self._fmt = str_format
+        self.value = raw_values
+        self.delimiter = delimiter
+
+    def to_str(self) -> str:
+        """Get stringified property representation."""
+        values = [int_fmt(v, self._fmt) for v in self.value]
+        return f"[{self.delimiter.join(values)}]"
 
 
 class BoolValue(PropertyValueBase):
@@ -300,11 +454,11 @@ class BoolValue(PropertyValueBase):
 
     def __init__(
         self,
-        tag: int,
-        raw_values: List[int],
-        true_values: Tuple[int] = (1,),
+        prop: PropertyTag,
+        raw_values: list[int],
+        true_values: tuple[int] = (1,),
         true_string: str = "YES",
-        false_values: Tuple[int] = (0,),
+        false_values: tuple[int] = (0,),
         false_string: str = "NO",
     ) -> None:
         """Initialize the Boolean-based property object.
@@ -316,7 +470,7 @@ class BoolValue(PropertyValueBase):
         :param false_values: Values representing 'False', defaults to (0,)
         :param false_string: String representing 'False, defaults to 'NO'
         """
-        super().__init__(tag)
+        super().__init__(prop)
         self._true_values = true_values
         self._true_string = true_string
         self._false_values = false_values
@@ -342,9 +496,9 @@ class EnumValue(PropertyValueBase):
 
     def __init__(
         self,
-        tag: int,
-        raw_values: List[int],
-        enum: Type[Enum],
+        prop: PropertyTag,
+        raw_values: list[int],
+        enum: Type[SpsdkEnum],
         na_msg: str = "Unknown Item",
     ) -> None:
         """Initialize the enumeration-based property object.
@@ -354,7 +508,7 @@ class EnumValue(PropertyValueBase):
         :param enum: Enumeration to pick from
         :param na_msg: Message to display if an item is not found in the enum
         """
-        super().__init__(tag)
+        super().__init__(prop)
         self._na_msg = na_msg
         self.enum = enum
         self.value = raw_values[0]
@@ -365,7 +519,10 @@ class EnumValue(PropertyValueBase):
 
     def to_str(self) -> str:
         """Get stringified property representation."""
-        return self.enum.name(self.value, f"{self._na_msg}: {self.value}")
+        try:
+            return self.enum.get_label(self.value)
+        except SPSDKKeyError:
+            return f"{self._na_msg}: {self.value}"
 
 
 class VersionValue(PropertyValueBase):
@@ -373,13 +530,13 @@ class VersionValue(PropertyValueBase):
 
     __slots__ = ("value",)
 
-    def __init__(self, tag: int, raw_values: List[int]) -> None:
+    def __init__(self, prop: PropertyTag, raw_values: list[int]) -> None:
         """Initialize the Version-based property object.
 
         :param tag: Property tag, see: `PropertyTag`
         :param raw_values: List of integers representing the property
         """
-        super().__init__(tag)
+        super().__init__(prop)
         self.value = Version(raw_values[0])
 
     def to_int(self) -> int:
@@ -396,24 +553,24 @@ class DeviceUidValue(PropertyValueBase):
 
     __slots__ = ("value",)
 
-    def __init__(self, tag: int, raw_values: List[int]) -> None:
+    def __init__(self, prop: PropertyTag, raw_values: list[int]) -> None:
         """Initialize the Version-based property object.
 
         :param tag: Property tag, see: `PropertyTag`
         :param raw_values: List of integers representing the property
         """
-        super().__init__(tag)
+        super().__init__(prop)
         self.value = b"".join(
-            [int.to_bytes(val, length=4, byteorder="little") for val in raw_values]
+            [int.to_bytes(val, length=4, byteorder=Endianness.LITTLE.value) for val in raw_values]
         )
 
     def to_int(self) -> int:
         """Get the raw integer portion of the property."""
-        return int.from_bytes(self.value, byteorder="big")
+        return int.from_bytes(self.value, byteorder=Endianness.BIG.value)
 
     def to_str(self) -> str:
         """Get stringified property representation."""
-        return " ".join(f"{item:02X}" for item in self.value)
+        return "".join(f"{item:02x}" for item in self.value)
 
 
 class ReservedRegionsValue(PropertyValueBase):
@@ -421,14 +578,14 @@ class ReservedRegionsValue(PropertyValueBase):
 
     __slots__ = ("regions",)
 
-    def __init__(self, tag: int, raw_values: List[int]) -> None:
+    def __init__(self, prop: PropertyTag, raw_values: list[int]) -> None:
         """Initialize the ReserverRegion-based property object.
 
         :param tag: Property tag, see: `PropertyTag`
         :param raw_values: List of integers representing the property
         """
-        super().__init__(tag)
-        self.regions: List[MemoryRegion] = []
+        super().__init__(prop)
+        self.regions: list[MemoryRegion] = []
         for i in range(0, len(raw_values), 2):
             if raw_values[i + 1] == 0:
                 continue
@@ -447,13 +604,13 @@ class AvailablePeripheralsValue(PropertyValueBase):
 
     __slots__ = ("value",)
 
-    def __init__(self, tag: int, raw_values: List[int]) -> None:
+    def __init__(self, prop: PropertyTag, raw_values: list[int]) -> None:
         """Initialize the AvailablePeripherals-based property object.
 
         :param tag: Property tag, see: `PropertyTag`
         :param raw_values: List of integers representing the property
         """
-        super().__init__(tag)
+        super().__init__(prop)
         self.value = raw_values[0]
 
     def to_int(self) -> int:
@@ -462,7 +619,13 @@ class AvailablePeripheralsValue(PropertyValueBase):
 
     def to_str(self) -> str:
         """Get stringified property representation."""
-        return ", ".join([key for key, value, _ in PeripheryTag if value & self.value])  # type: ignore
+        return ", ".join(
+            [
+                peripheral_tag.label
+                for peripheral_tag in PeripheryTag
+                if peripheral_tag.tag & self.value
+            ]
+        )
 
 
 class AvailableCommandsValue(PropertyValueBase):
@@ -471,21 +634,21 @@ class AvailableCommandsValue(PropertyValueBase):
     __slots__ = ("value",)
 
     @property
-    def tags(self) -> List[str]:
+    def tags(self) -> list[int]:
         """List of tags representing Available commands."""
         return [
-            tag_value  # type: ignore
-            for _, tag_value, _ in CommandTag  # type: ignore
-            if tag_value > 0 and (1 << tag_value - 1) & self.value  # type: ignore
+            cmd_tag.tag
+            for cmd_tag in CommandTag
+            if cmd_tag.tag > 0 and (1 << cmd_tag.tag - 1) & self.value
         ]
 
-    def __init__(self, tag: int, raw_values: List[int]) -> None:
+    def __init__(self, prop: PropertyTag, raw_values: list[int]) -> None:
         """Initialize the AvailableCommands-based property object.
 
         :param tag: Property tag, see: `PropertyTag`
         :param raw_values: List of integers representing the property
         """
-        super().__init__(tag)
+        super().__init__(prop)
         self.value = raw_values[0]
 
     def __contains__(self, item: int) -> bool:
@@ -494,9 +657,9 @@ class AvailableCommandsValue(PropertyValueBase):
     def to_str(self) -> str:
         """Get stringified property representation."""
         return [
-            name  # type: ignore
-            for name, value, _ in CommandTag  # type: ignore
-            if value > 0 and (1 << value - 1) & self.value  # type: ignore
+            cmd_tag.label  # type: ignore
+            for cmd_tag in CommandTag
+            if cmd_tag.tag > 0 and (1 << cmd_tag.tag - 1) & self.value
         ]
 
 
@@ -518,15 +681,15 @@ class IrqNotifierPinValue(PropertyValueBase):
     @property
     def enabled(self) -> bool:
         """Indicates whether IRQ reporting is enabled."""
-        return bool(self.value & (1 << 32))
+        return bool(self.value & (1 << 31))
 
-    def __init__(self, tag: int, raw_values: List[int]) -> None:
+    def __init__(self, prop: PropertyTag, raw_values: list[int]) -> None:
         """Initialize the IrqNotifierPin-based property object.
 
         :param tag: Property tag, see: `PropertyTag`
         :param raw_values: List of integers representing the property
         """
-        super().__init__(tag)
+        super().__init__(prop)
         self.value = raw_values[0]
 
     def __bool__(self) -> bool:
@@ -552,22 +715,24 @@ class ExternalMemoryAttributesValue(PropertyValueBase):
         "block_size",
     )
 
-    def __init__(self, tag: int, raw_values: List[int], mem_id: int = 0) -> None:
+    def __init__(self, prop: PropertyTag, raw_values: list[int], mem_id: int = 0) -> None:
         """Initialize the ExternalMemoryAttributes-based property object.
 
         :param tag: Property tag, see: `PropertyTag`
         :param raw_values: List of integers representing the property
         :param mem_id: ID of the external memory
         """
-        super().__init__(tag)
+        super().__init__(prop)
         self.mem_id = mem_id
-        self.start_address = raw_values[1] if raw_values[0] & ExtMemPropTags.START_ADDRESS else None
-        self.total_size = (
-            raw_values[2] * 1024 if raw_values[0] & ExtMemPropTags.SIZE_IN_KBYTES else None
+        self.start_address = (
+            raw_values[1] if raw_values[0] & ExtMemPropTags.START_ADDRESS.tag else None
         )
-        self.page_size = raw_values[3] if raw_values[0] & ExtMemPropTags.PAGE_SIZE else None
-        self.sector_size = raw_values[4] if raw_values[0] & ExtMemPropTags.SECTOR_SIZE else None
-        self.block_size = raw_values[5] if raw_values[0] & ExtMemPropTags.BLOCK_SIZE else None
+        self.total_size = (
+            raw_values[2] * 1024 if raw_values[0] & ExtMemPropTags.SIZE_IN_KBYTES.tag else None
+        )
+        self.page_size = raw_values[3] if raw_values[0] & ExtMemPropTags.PAGE_SIZE.tag else None
+        self.sector_size = raw_values[4] if raw_values[0] & ExtMemPropTags.SECTOR_SIZE.tag else None
+        self.block_size = raw_values[5] if raw_values[0] & ExtMemPropTags.BLOCK_SIZE.tag else None
         self.value = raw_values[0]
 
     def to_str(self) -> str:
@@ -586,8 +751,25 @@ class ExternalMemoryAttributesValue(PropertyValueBase):
         return ", ".join(str_values)
 
 
+class FuseLock:
+    """Fuse Lock."""
+
+    def __init__(self, index: int, locked: bool) -> None:
+        """Initialize object representing information about fuse lock.
+
+        :param index: value of OTP index
+        :param locked: status of the lock, true if locked
+        """
+        self.index = index
+        self.locked = locked
+
+    def __str__(self) -> str:
+        status = "LOCKED" if self.locked else "UNLOCKED"
+        return f"  FUSE{(self.index):03d}: {status}\r\n"
+
+
 class FuseLockRegister:
-    """RAM memory regions."""
+    """Fuse Lock Register."""
 
     def __init__(self, value: int, index: int, start: int = 0) -> None:
         """Initialize object representing the OTP Controller Program Locked Status.
@@ -600,32 +782,35 @@ class FuseLockRegister:
         self.value = value
         self.index = index
         self.msg = ""
+        self.bitfields: list[FuseLock] = []
 
         shift = 0
         for _ in range(start, 32):
-            bit = (value >> shift) & 1
-            status = "LOCKED" if bit else "UNLOCKED"
-            self.msg += f"  FUSE{(index + shift):03d}: {status}\r\n"
+            locked = (value >> shift) & 1
+            self.bitfields.append(FuseLock(index + shift, bool(locked)))
             shift += 1
 
     def __str__(self) -> str:
         """Get stringified property representation."""
+        if self.bitfields:
+            for bitfield in self.bitfields:
+                self.msg += str(bitfield)
         return f"\r\n{self.msg}"
 
 
 class FuseLockedStatus(PropertyValueBase):
-    """Reserver Regions property."""
+    """Class representing FuseLocked registers."""
 
     __slots__ = ("fuses",)
 
-    def __init__(self, tag: int, raw_values: List[int]) -> None:
+    def __init__(self, prop: PropertyTag, raw_values: list[int]) -> None:
         """Initialize the FuseLockedStatus property object.
 
         :param tag: Property tag, see: `PropertyTag`
         :param raw_values: List of integers representing the property
         """
-        super().__init__(tag)
-        self.fuses: List[FuseLockRegister] = []
+        super().__init__(prop)
+        self.fuses: list[FuseLockRegister] = []
         idx = 0
         for count, val in enumerate(raw_values):
             start = 0
@@ -643,12 +828,84 @@ class FuseLockedStatus(PropertyValueBase):
             msg += f"OTP Controller Program Locked Status {count} Register: {register}"
         return msg
 
+    def get_fuses(self) -> list[FuseLock]:
+        """Get list of fuses bitfield objects.
+
+        :return: list of FuseLockBitfield objects
+        """
+        fuses = []
+        for registers in self.fuses:
+            fuses.extend(registers.bitfields)
+        return fuses
+
+
+class SHEFlashPartition(PropertyValueBase):
+    """Class representing SHE Flash Partition property."""
+
+    __slots__ = ("max_keys", "flash_size")
+
+    def __init__(self, prop: PropertyTag, raw_values: list[int]) -> None:
+        """Initialize the SHE Flash Partition property object.
+
+        :param tag: Property tag, see: `PropertyTag`
+        :param raw_values: List of integers representing the property
+        """
+        super().__init__(prop)
+        self.max_keys = raw_values[0] & 0x03
+        self.flash_size = (raw_values[0] >> 8) & 0x03
+
+    def to_str(self) -> str:
+        """Get stringified property representation."""
+        max_keys_mapping = {
+            0: "0 Keys, CSEc disabled",
+            1: "max 5 Key",
+            2: "max 10 Keys",
+            3: "max 20 Keys",
+        }
+        flash_size_mapping = {
+            0: "64kB",
+            1: "48kB",
+            2: "32kB",
+            3: "0kB",
+        }
+        return (
+            f"{flash_size_mapping[self.flash_size]} EEPROM "
+            f"with {max_keys_mapping[self.max_keys]}"
+        )
+
+
+class SHEBootMode(PropertyValueBase):
+    """Class representing SHE Boot Mode property."""
+
+    __slots__ = ("size", "mode")
+
+    def __init__(self, prop: PropertyTag, raw_values: list[int]) -> None:
+        """Initialize the SHE Boot Mode property object.
+
+        :param tag: Property tag, see: `PropertyTag`
+        :param raw_values: List of integers representing the property
+        """
+        super().__init__(prop)
+        self.size = raw_values[0] & 0x3FFF_FFFF
+        self.mode = (raw_values[0] >> 30) & 0x03
+
+    def to_str(self) -> str:
+        """Get stringified property representation."""
+        mode_mapping = {0: "Strict Boot", 1: "Serial Boot", 2: "Parallel Boot", 3: "Undefined"}
+        return (
+            f"SHE Boot Mode: {mode_mapping.get(self.mode, 'Unknown')} ({self.mode})\n"
+            f"SHE Boot Size: {size_fmt(self.size // 8)} (0x{self.size:_x})"
+        )
+
+    def __str__(self) -> str:
+        return self.to_str()
+
 
 ########################################################################################################################
 # McuBoot property response parser
 ########################################################################################################################
 
-PROPERTIES = {
+PROPERTY_RESPONSE: dict[PropertyTag, dict] = {
     PropertyTag.CURRENT_VERSION: {"class": VersionValue, "kwargs": {}},
     PropertyTag.AVAILABLE_PERIPHERALS: {
         "class": AvailablePeripheralsValue,
@@ -742,42 +999,48 @@ PROPERTIES = {
         "class": FuseLockedStatus,
         "kwargs": {},
     },
-}
-
-PROPERTIES_KW45XX = {
-    PropertyTagKw45xx.VERIFY_ERASE: {
-        "class": BoolValue,
-        "kwargs": {"true_string": "ENABLE", "false_string": "DISABLE"},
-    },
-    PropertyTagKw45xx.BOOT_STATUS_REGISTER: {
+    PropertyTag.BOOT_STATUS_REGISTER: {
         "class": IntValue,
         "kwargs": {"str_format": "int32"},
     },
-    PropertyTagKw45xx.FIRMWARE_VERSION: {
+    PropertyTag.FIRMWARE_VERSION: {
         "class": IntValue,
         "kwargs": {"str_format": "int32"},
     },
-    PropertyTagKw45xx.FUSE_PROGRAM_VOLTAGE: {
+    PropertyTag.FUSE_PROGRAM_VOLTAGE: {
         "class": BoolValue,
         "kwargs": {
             "true_string": "Over Drive Voltage (2.5 V)",
             "false_string": "Normal Voltage (1.8 V)",
         },
     },
+    PropertyTag.SHE_FLASH_PARTITION: {"class": SHEFlashPartition, "kwargs": {}},
+    PropertyTag.SHE_BOOT_MODE: {"class": SHEBootMode, "kwargs": {}},
+    PropertyTag.VERIFY_ERASE: {
+        "class": BoolValue,
+        "kwargs": {"true_string": "ENABLE", "false_string": "DISABLE"},
+    },
+    PropertyTag.LIFE_CYCLE_STATE: {
+        "class": BoolValue,
+        "kwargs": {
+            "true_values": (0x00000000, 0x5AA55AA5),
+            "true_string": "development life cycle",
+            "false_values": (0x00000001, 0xC33CC33C),
+            "false_string": "deployment life cycle",
+        },
+    },
+    PropertyTag.UNKNOWN: {
+        "class": IntListValue,
+        "kwargs": {"str_format": "hex"},
+    },
 }
 
-PROPERTIES_OVERRIDE = {"kw45xx": PROPERTIES_KW45XX}
-PROPERTIES_OVERRIDE = {"k32w1xx": PROPERTIES_KW45XX}
-
-PROPERTY_TAG_OVERRIDE = {"kw45xx": PropertyTagKw45xx}
-PROPERTY_TAG_OVERRIDE = {"k32w1xx": PropertyTagKw45xx}
-
 
 def parse_property_value(
-    property_tag: int,
-    raw_values: List[int],
+    property_tag: Union[int, PropertyTag],
+    raw_values: list[int],
     ext_mem_id: Optional[int] = None,
-    family: Optional[str] = None,
+    family: Optional[FamilyRevision] = None,
 ) -> Optional[PropertyValueBase]:
     """Parse the property value received from the device.
 
@@ -787,19 +1050,35 @@ def parse_property_value(
     :param family: supported family
     :return: Object representing the property
     """
-    assert isinstance(property_tag, int)
     assert isinstance(raw_values, list)
-    properties_dict = deepcopy(PROPERTIES)
-    if family:
-        properties_dict.update(PROPERTIES_OVERRIDE[family])  # type: ignore
-    if property_tag not in properties_dict.keys():
-        return None
-    cls = properties_dict[property_tag]["class"]  # type: ignore
-    kwargs: dict = properties_dict[property_tag]["kwargs"]  # type: ignore
+    properties_dict = get_properties(family)
+    if isinstance(property_tag, int):
+        if property_tag in list(properties_dict.keys()):
+            property_tag = next(
+                prop for idx, prop in properties_dict.items() if idx == property_tag
+            )
+        else:
+            property_tag = PropertyTag.UNKNOWN
+
+    property_response = PROPERTY_RESPONSE[property_tag]
+    cls: Callable = property_response["class"]
+    kwargs: dict = property_response["kwargs"]
     if "mem_id" in kwargs:
-        kwargs["mem_id"] = ext_mem_id  # type: ignore
-    obj = cls(property_tag, raw_values, **kwargs)  # type: ignore
-    if family:
-        obj.name = PROPERTY_TAG_OVERRIDE[family].name(property_tag)
-        obj.desc = PROPERTY_TAG_OVERRIDE[family].desc(property_tag)
-    return obj  # type: ignore
+        kwargs["mem_id"] = ext_mem_id
+    obj = cls(property_tag, raw_values, **kwargs)
+    return obj
+
+
+def get_property_tag_label(
+    mboot_property: Union[PropertyTag, int], family: Optional[FamilyRevision] = None
+) -> tuple[int, str]:
+    """Get property tag and label."""
+    if isinstance(mboot_property, int):
+        try:
+            prop = PropertyTag.from_index(mboot_property, family)
+            return mboot_property, prop.label
+        except SPSDKKeyError:
+            logger.warning(f"Unknown property id: {mboot_property} ({hex(mboot_property)})")
+            return mboot_property, "Unknown"
+
+    return get_property_index(mboot_property, family), mboot_property.label
