@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2020-2021 NXP
+# Copyright 2020-2024 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
@@ -10,11 +10,16 @@
 import ast
 import logging
 import re
-from typing import Tuple
+import sys
 
-import astunparse
+if sys.version_info < (3, 9):
+    from astunparse import unparse
+else:
+    from ast import unparse
 
-from .translator import Translator
+from spsdk.pfr.translator import Translator
+
+logger = logging.getLogger(__name__)
 
 
 class MyTransformer(ast.NodeTransformer):
@@ -26,12 +31,12 @@ class MyTransformer(ast.NodeTransformer):
         :param translator: Translator instance
         """
         self.translator = translator
-        self.logger = logging.getLogger("transformer")
+        self.logger = logger.getChild("transformer")
 
     def visit_Attribute(self, node: ast.Attribute) -> ast.Constant:  # pylint: disable=invalid-name
         """Translate Attribute Nodes."""
         self.logger.debug("Transforming node attribute...")
-        thing = astunparse.unparse(node).strip()
+        thing = unparse(node).strip()
         value = self.translator.translate(thing)
         self.logger.debug(f"Attribute '{thing}' transformed into {value:x}")
         result = ast.Constant(value=value, kind=None)
@@ -53,23 +58,23 @@ class Processor:
 
         :param translator: Translator instance
         """
-        self.logger = logging.getLogger("processor")
+        self.logger = logger.getChild("processor")
         self.transformer = MyTransformer(translator)
 
-    def process(self, condition: str) -> Tuple[bool, str]:
+    def process(self, condition: str) -> tuple[bool, str]:
         """Process individual condition from rules.
 
         :param condition: condition to quantify
-        :return: Boolean result and values for translated keys
+        :return: Tuple with boolean result and string with translated keys
         """
         self.logger.debug(f"Transforming condition: {condition}")
         org_node = ast.parse(condition, mode="eval")
         new_node = self.transformer.visit(org_node)
-        self.logger.debug(f"Transformed condition: {new_node}")
-        node_str = astunparse.unparse(new_node)
+        node_str = unparse(new_node).rstrip()
+        self.logger.debug(f"Transformed condition: {node_str}")
         node_str = self._replace_int_as_hex(node_str)
         # pylint: disable=eval-used
-        result = eval(compile(new_node, filename="", mode="eval"))
+        result = eval(compile(new_node, filename="", mode="eval"))  # nosec
         return result, node_str
 
     @staticmethod
