@@ -1,31 +1,32 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2019-2021 NXP
+# Copyright 2019-2024 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
 """Module implementing SBFile version 1."""
-
-from spsdk.mboot import ExtMemId
-from .headers import SecureBootFlagsV1
-from .images import SecureBootV1
-from .sections import BootSectionV1
-from ..commands import (
-    CmdFill,
-    CmdTag,
-    CmdNop,
-    CmdMemEnable,
-    CmdErase,
-    CmdReset,
+from spsdk.sbfile.misc import BcdVersion3
+from spsdk.sbfile.sb1.headers import SecureBootFlagsV1
+from spsdk.sbfile.sb1.images import SecureBootV1
+from spsdk.sbfile.sb1.sections import BootSectionV1
+from spsdk.sbfile.sb2 import commands
+from spsdk.sbfile.sb2.commands import (
     CmdCall,
-    CmdLoad,
+    CmdErase,
+    CmdFill,
     CmdJump,
+    CmdLoad,
+    CmdMemEnable,
+    CmdNop,
     CmdProg,
+    CmdReset,
+    CmdTag,
 )
-from ..misc import BcdVersion3
 
 __all__ = [
+    # modules
+    "commands",
     # main classes
     "SecureBootV1",
     "BootSectionV1",
@@ -43,5 +44,4 @@ __all__ = [
     # helper classes and enums
     "SecureBootFlagsV1",
     "BcdVersion3",
-    "ExtMemId",
 ]
