@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2019-2023 NXP
+# Copyright 2019-2024 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
@@ -9,16 +9,17 @@
 import math
 from abc import abstractmethod
 from struct import calcsize, pack, unpack_from
-from typing import Any, Mapping, Optional, Type
+from typing import Mapping, Optional, Type
 
-from crcmod.predefined import mkPredefinedCrcFun
+from typing_extensions import Self
 
-from spsdk import SPSDKError
-from spsdk.mboot import ExtMemId, MemId
+from spsdk.crypto.crc import CrcAlg, from_crc_algorithm
+from spsdk.exceptions import SPSDKError
+from spsdk.mboot.memories import ExtMemId
 from spsdk.sbfile.misc import SecBootBlckSize
-from spsdk.utils.crypto.abstract import BaseClass
-from spsdk.utils.easy_enum import Enum
-from spsdk.utils.misc import DebugInfo, swap16
+from spsdk.utils.abstract import BaseClass
+from spsdk.utils.misc import Endianness
+from spsdk.utils.spsdk_enum import SpsdkEnum
 
 ########################################################################################################################
 # Constants
@@ -29,39 +30,44 @@ DEVICE_ID_SHIFT = 0
 GROUP_ID_MASK = 0xF00
 GROUP_ID_SHIFT = 8
 
+
 ########################################################################################################################
 # Enums
 ########################################################################################################################
-class EnumCmdTag(Enum):
+class EnumCmdTag(SpsdkEnum):
     """Command tags."""
 
-    NOP = 0x0
-    TAG = 0x1
-    LOAD = 0x2
-    FILL = 0x3
-    JUMP = 0x4
-    CALL = 0x5
-    ERASE = 0x7
-    RESET = 0x8
-    MEM_ENABLE = 0x9
-    PROG = 0xA
-    FW_VERSION_CHECK = (0xB, "Check FW version fuse value")
-    WR_KEYSTORE_TO_NV = (0xC, "Restore key-store restore to non-volatile memory")
-    WR_KEYSTORE_FROM_NV = (0xD, "Backup key-store from non-volatile memory")
+    NOP = (0x0, "NOP")
+    TAG = (0x1, "TAG")
+    LOAD = (0x2, "LOAD")
+    FILL = (0x3, "FILL")
+    JUMP = (0x4, "JUMP")
+    CALL = (0x5, "CALL")
+    ERASE = (0x7, "ERASE")
+    RESET = (0x8, "RESET")
+    MEM_ENABLE = (0x9, "MEM_ENABLE")
+    PROG = (0xA, "PROG")
+    FW_VERSION_CHECK = (0xB, "FW_VERSION_CHECK", "Check FW version fuse value")
+    WR_KEYSTORE_TO_NV = (
+        0xC,
+        "WR_KEYSTORE_TO_NV",
+        "Restore key-store restore to non-volatile memory",
+    )
+    WR_KEYSTORE_FROM_NV = (0xD, "WR_KEYSTORE_FROM_NV", "Backup key-store from non-volatile memory")
 
 
-class EnumSectionFlag(Enum):
+class EnumSectionFlag(SpsdkEnum):
     """Section flags."""
 
-    BOOTABLE = 0x0001
-    CLEARTEXT = 0x0002
-    LAST_SECT = 0x8000
+    BOOTABLE = (0x0001, "BOOTABLE")
+    CLEARTEXT = (0x0002, "CLEARTEXT")
+    LAST_SECT = (0x8000, "LAST_SECT")
 
 
 ########################################################################################################################
 # Header Class
 ########################################################################################################################
-class CmdHeader:
+class CmdHeader(BaseClass):
     """SBFile command header."""
 
     FORMAT = "<2BH3L"
@@ -76,7 +82,7 @@ class CmdHeader:
             checksum = (checksum + raw_data[i]) & 0xFF
         return checksum
 
-    def __init__(self, tag: int, flags: int = 0) -> None:
+    def __init__(self, tag: int, flags: int = 0, zero_filling: bool = False) -> None:
         """Initialize header."""
         if tag not in EnumCmdTag.tags():
             raise SPSDKError("Incorrect command tag")
@@ -85,15 +91,15 @@ class CmdHeader:
         self.address = 0
         self.count = 0
         self.data = 0
+        self.zero_filling = zero_filling
 
-    def __eq__(self, obj: Any) -> bool:
-        return isinstance(obj, self.__class__) and vars(obj) == vars(self)
-
-    def __ne__(self, obj: Any) -> bool:
-        return not self.__eq__(obj)
+    def __repr__(self) -> str:
+        return f"SB2 Command header, TAG:{self.tag}"
 
     def __str__(self) -> str:
-        tag = EnumCmdTag.get(self.tag, f"0x{self.tag:02X}")
+        tag = (
+            EnumCmdTag.get_label(self.tag) if self.tag in EnumCmdTag.tags() else f"0x{self.tag:02X}"
+        )
         return (
             f"tag={tag}, flags=0x{self.flags:04X}, "
             f"address=0x{self.address:08X}, count=0x{self.count:08X}, data=0x{self.data:08X}"
@@ -112,21 +118,18 @@ class CmdHeader:
         return self._raw_data(self.crc)
 
     @classmethod
-    def parse(cls, data: bytes, offset: int = 0) -> "CmdHeader":
+    def parse(cls, data: bytes) -> Self:
         """Parse command header from bytes.
 
         :param data: Input data as bytes
-        :param offset: The offset of input data
         :return: CMDHeader object
-        :raise Exception: raised when size is incorrect
+        :raises SPSDKError: raised when size is incorrect
         :raises SPSDKError: Raised when CRC is incorrect
         """
-        if calcsize(cls.FORMAT) > len(data) - offset:
-            raise Exception()
-        obj = cls(EnumCmdTag.NOP)
-        (crc, obj.tag, obj.flags, obj.address, obj.count, obj.data) = unpack_from(
-            cls.FORMAT, data, offset
-        )
+        if calcsize(cls.FORMAT) > len(data):
+            raise SPSDKError("Incorrect size")
+        obj = cls(EnumCmdTag.NOP.tag)
+        (crc, obj.tag, obj.flags, obj.address, obj.count, obj.data) = unpack_from(cls.FORMAT, data)
         if crc != obj.crc:
             raise SPSDKError("CRC does not match")
         return obj
@@ -147,9 +150,9 @@ class CmdBaseClass(BaseClass):
     # shift for group ID inside flags
     ROM_MEM_GROUP_ID_SHIFT = 4
 
-    def __init__(self, tag: int) -> None:
+    def __init__(self, tag: EnumCmdTag) -> None:
         """Initialize CmdBase."""
-        self._header = CmdHeader(tag)
+        self._header = CmdHeader(tag.tag)
 
     @property
     def header(self) -> CmdHeader:
@@ -161,24 +164,16 @@ class CmdBaseClass(BaseClass):
         """Return size of the command in binary format (including header)."""
         return CmdHeader.SIZE  # this is default implementation
 
-    def __str__(self) -> str:
+    def __repr__(self) -> str:
         return "Command: " + str(self._header)  # default implementation: use command name
 
-    def info(self) -> str:
+    def __str__(self) -> str:
         """Return text info about the instance."""
-        return str(self) + "\n"  # default implementation is same as __str__
+        return repr(self) + "\n"  # default implementation is same as __repr__
 
-    def export(self, dbg_info: DebugInfo = DebugInfo.disabled()) -> bytes:
+    def export(self) -> bytes:
         """Return object serialized into bytes."""
-        dbg_info.append_section("Command:" + EnumCmdTag.name(self.header.tag))
-        cmd_data = self._header.export()  # default implementation
-        dbg_info.append_binary_data("cmd-header", cmd_data)
-        return cmd_data
-
-    @classmethod
-    @abstractmethod
-    def parse(cls, data: bytes, offset: int = 0) -> "CmdBaseClass":
-        """Deserialize object from binary."""
+        return self._header.export()  # default implementation
 
 
 class CmdNop(CmdBaseClass):
@@ -189,15 +184,14 @@ class CmdNop(CmdBaseClass):
         super().__init__(EnumCmdTag.NOP)
 
     @classmethod
-    def parse(cls, data: bytes, offset: int = 0) -> "CmdNop":
+    def parse(cls, data: bytes) -> Self:
         """Parse command from bytes.
 
         :param data: Input data as bytes
-        :param offset: The offset of input data
         :return: CMD Nop object
         :raises SPSDKError: When there is incorrect header tag
         """
-        header = CmdHeader.parse(data, offset)
+        header = CmdHeader.parse(data)
         if header.tag != EnumCmdTag.NOP:
             raise SPSDKError("Incorrect header tag")
         return cls()
@@ -214,15 +208,14 @@ class CmdTag(CmdBaseClass):
         super().__init__(EnumCmdTag.TAG)
 
     @classmethod
-    def parse(cls, data: bytes, offset: int = 0) -> "CmdTag":
+    def parse(cls, data: bytes) -> Self:
         """Parse command from bytes.
 
         :param data: Input data as bytes
-        :param offset: The offset of input data
         :return: parsed instance
         :raises SPSDKError: When there is incorrect header tag
         """
-        header = CmdHeader.parse(data, offset)
+        header = CmdHeader.parse(data)
         if header.tag != EnumCmdTag.TAG:
             raise SPSDKError("Incorrect header tag")
         result = cls()
@@ -267,7 +260,9 @@ class CmdLoad(CmdBaseClass):
             size += CmdHeader.SIZE - (size % CmdHeader.SIZE)
         return size
 
-    def __init__(self, address: int, data: bytes, mem_id: int = 0) -> None:
+    def __init__(
+        self, address: int, data: bytes, mem_id: int = 0, zero_filling: bool = False
+    ) -> None:
         """Initialize CMD Load."""
         super().__init__(EnumCmdTag.LOAD)
         assert isinstance(data, (bytes, bytearray))
@@ -286,51 +281,53 @@ class CmdLoad(CmdBaseClass):
             (group_id << self.ROM_MEM_GROUP_ID_SHIFT) & self.ROM_MEM_GROUP_ID_MASK
         )
 
+        self.zero_filling = zero_filling
+
     def __str__(self) -> str:
         return (
             f"LOAD: Address=0x{self.address:08X}, DataLen={len(self.data)}, "
             f"Flags=0x{self.flags:08X}, MemId=0x{self.mem_id:08X}"
         )
 
-    def export(self, dbg_info: DebugInfo = DebugInfo.disabled()) -> bytes:
+    def export(self) -> bytes:
         """Export command as binary."""
         self._update_data()
-        result = super().export(dbg_info)
-        dbg_info.append_binary_section("load-data", self.data)
+        result = super().export()
         return result + self.data
 
     def _update_data(self) -> None:
         """Update command data."""
         # padding data
-        self.data = SecBootBlckSize.align_block_fill_random(self.data)
+        if self.zero_filling or self._header.zero_filling:
+            self.data = SecBootBlckSize.align_block_fill_zeros(self.data)
+        else:
+            self.data = SecBootBlckSize.align_block_fill_random(self.data)
         # update header
         self._header.count = len(self.data)
-        crc32_function = mkPredefinedCrcFun("crc-32-mpeg")
-        self._header.data = crc32_function(self.data, 0xFFFFFFFF)
+        crc_ob = from_crc_algorithm(CrcAlg.CRC32_MPEG)
+        self._header.data = crc_ob.calculate(self.data)
 
     @classmethod
-    def parse(cls, data: bytes, offset: int = 0) -> "CmdLoad":
+    def parse(cls, data: bytes) -> Self:
         """Parse command from bytes.
 
         :param data: Input data as bytes
-        :param offset: The offset of input data
         :return: CMD Load object
         :raises SPSDKError: Raised when there is invalid CRC
         :raises SPSDKError: When there is incorrect header tag
         """
-        header = CmdHeader.parse(data, offset)
+        header = CmdHeader.parse(data)
         if header.tag != EnumCmdTag.LOAD:
             raise SPSDKError("Incorrect header tag")
-        offset += CmdHeader.SIZE
         header_count = SecBootBlckSize.align(header.count)
-        cmd_data = data[offset : offset + header_count]
-        crc32_function = mkPredefinedCrcFun("crc-32-mpeg")
-        if header.data != crc32_function(cmd_data, 0xFFFFFFFF):
+        cmd_data = data[CmdHeader.SIZE : CmdHeader.SIZE + header_count]
+        crc_obj = from_crc_algorithm(CrcAlg.CRC32_MPEG)
+        if header.data != crc_obj.calculate(cmd_data):
             raise SPSDKError("Invalid CRC in the command header")
         device_id = (header.flags & cls.ROM_MEM_DEVICE_ID_MASK) >> cls.ROM_MEM_DEVICE_ID_SHIFT
         group_id = (header.flags & cls.ROM_MEM_GROUP_ID_MASK) >> cls.ROM_MEM_GROUP_ID_SHIFT
         mem_id = get_memory_id(device_id, group_id)
-        obj = CmdLoad(header.address, cmd_data, mem_id)
+        obj = cls(header.address, cmd_data, mem_id)
         obj.header.data = header.data
         obj.header.flags = header.flags
         obj._update_data()
@@ -363,7 +360,9 @@ class CmdFill(CmdBaseClass):
             size += CmdHeader.SIZE - (size % CmdHeader.SIZE)
         return size
 
-    def __init__(self, address: int, pattern: int, length: Optional[int] = None) -> None:
+    def __init__(
+        self, address: int, pattern: int, length: Optional[int] = None, zero_filling: bool = False
+    ) -> None:
         """Initialize Command Fill.
 
         :param address: to write data
@@ -384,7 +383,7 @@ class CmdFill(CmdBaseClass):
         # the length to 4 bytes with the first byte being zero.
         if 3 == math.ceil(pattern_len):
             pattern_len = 4
-        pattern_bytes = pattern.to_bytes(math.ceil(pattern_len), "big")
+        pattern_bytes = pattern.to_bytes(math.ceil(pattern_len), Endianness.BIG.value)
         # The pattern length is computed above, but as we transform the number
         # into bytes, compute the len again just in case - a bit paranoid
         # approach chosen.
@@ -397,6 +396,7 @@ class CmdFill(CmdBaseClass):
         # update header
         self._header.data = unpack_from(">L", self._pattern)[0]
         self._header.count = length
+        self.zero_filling = zero_filling
 
     @property
     def pattern(self) -> bytes:
@@ -408,28 +408,28 @@ class CmdFill(CmdBaseClass):
             f"{byte:02X}" for byte in self._pattern
         )
 
-    def export(self, dbg_info: DebugInfo = DebugInfo.disabled()) -> bytes:
+    def export(self) -> bytes:
         """Return command in binary form (serialization)."""
         # export cmd
-        data = super().export(dbg_info)
+        data = super().export()
         # export additional data
-        data = SecBootBlckSize.align_block_fill_random(data)
+        if self.zero_filling or self._header.zero_filling:
+            data = SecBootBlckSize.align_block_fill_zeros(data)
+        else:
+            data = SecBootBlckSize.align_block_fill_random(data)
         return data
 
     @classmethod
-    def parse(cls, data: bytes, offset: int = 0) -> "CmdFill":
+    def parse(cls, data: bytes) -> Self:
         """Parse command from bytes.
 
         :param data: Input data as bytes
-        :param offset: The offset of input data
         :return: Command Fill object
         :raises SPSDKError: If incorrect header tag
         """
-        header = CmdHeader.parse(data, offset)
+        header = CmdHeader.parse(data)
         if header.tag != EnumCmdTag.FILL:
             raise SPSDKError("Incorrect header tag")
-        # The last 4 bytes of header are part of pattern value
-        offset += CmdHeader.SIZE - 4
         return cls(header.address, header.data, header.count)
 
 
@@ -490,15 +490,14 @@ class CmdJump(CmdBaseClass):
         return nfo
 
     @classmethod
-    def parse(cls, data: bytes, offset: int = 0) -> "CmdJump":
+    def parse(cls, data: bytes) -> Self:
         """Parse command from bytes.
 
         :param data: Input data as bytes
-        :param offset: The offset of input data
         :return: Command Jump object
         :raises SPSDKError: If incorrect header tag
         """
-        header = CmdHeader.parse(data, offset)
+        header = CmdHeader.parse(data)
         if header.tag != EnumCmdTag.JUMP:
             raise SPSDKError("Incorrect header tag")
         return cls(header.address, header.data, header.count if header.flags else None)
@@ -543,15 +542,14 @@ class CmdCall(CmdBaseClass):
         return f"CALL: Address=0x{self.address:08X}, Argument=0x{self.argument:08X}"
 
     @classmethod
-    def parse(cls, data: bytes, offset: int = 0) -> "CmdCall":
+    def parse(cls, data: bytes) -> Self:
         """Parse command from bytes.
 
         :param data: Input data as bytes
-        :param offset: The offset of input data
         :return: Command Call object
         :raises SPSDKError: If incorrect header tag
         """
-        header = CmdHeader.parse(data, offset)
+        header = CmdHeader.parse(data)
         if header.tag != EnumCmdTag.CALL:
             raise SPSDKError("Incorrect header tag")
         return cls(header.address, header.data)
@@ -618,15 +616,14 @@ class CmdErase(CmdBaseClass):
         )
 
     @classmethod
-    def parse(cls, data: bytes, offset: int = 0) -> "CmdErase":
+    def parse(cls, data: bytes) -> Self:
         """Parse command from bytes.
 
         :param data: Input data as bytes
-        :param offset: The offset of input data
         :return: Command Erase object
         :raises SPSDKError: If incorrect header tag
         """
-        header = CmdHeader.parse(data, offset)
+        header = CmdHeader.parse(data)
         if header.tag != EnumCmdTag.ERASE:
             raise SPSDKError("Invalid header tag")
         device_id = (header.flags & cls.ROM_MEM_DEVICE_ID_MASK) >> cls.ROM_MEM_DEVICE_ID_SHIFT
@@ -643,15 +640,14 @@ class CmdReset(CmdBaseClass):
         super().__init__(EnumCmdTag.RESET)
 
     @classmethod
-    def parse(cls, data: bytes, offset: int = 0) -> "CmdReset":
+    def parse(cls, data: bytes) -> Self:
         """Parse command from bytes.
 
         :param data: Input data as bytes
-        :param offset: The offset of input data
         :return: Cmd Reset object
         :raises SPSDKError: If incorrect header tag
         """
-        header = CmdHeader.parse(data, offset)
+        header = CmdHeader.parse(data)
         if header.tag != EnumCmdTag.RESET:
             raise SPSDKError("Invalid header tag")
         return cls()
@@ -720,15 +716,14 @@ class CmdMemEnable(CmdBaseClass):
         )
 
     @classmethod
-    def parse(cls, data: bytes, offset: int = 0) -> "CmdMemEnable":
+    def parse(cls, data: bytes) -> Self:
         """Parse command from bytes.
 
         :param data: Input data as bytes
-        :param offset: The offset of input data
         :return: Command Memory Enable object
         :raises SPSDKError: If incorrect header tag
         """
-        header = CmdHeader.parse(data, offset)
+        header = CmdHeader.parse(data)
         if header.tag != EnumCmdTag.MEM_ENABLE:
             raise SPSDKError("Invalid header tag")
         device_id = (header.flags & cls.ROM_MEM_DEVICE_ID_MASK) >> cls.ROM_MEM_DEVICE_ID_SHIFT
@@ -830,26 +825,25 @@ class CmdProg(CmdBaseClass):
         )
 
     @classmethod
-    def parse(cls, data: bytes, offset: int = 0) -> "CmdProg":
+    def parse(cls, data: bytes) -> Self:
         """Parse command from bytes.
 
         :param data: Input data as bytes
-        :param offset: The offset of input data
         :return: parsed command object
         :raises SPSDKError: If incorrect header tag
         """
-        header = CmdHeader.parse(data, offset)
+        header = CmdHeader.parse(data)
         if header.tag != EnumCmdTag.PROG:
             raise SPSDKError("Invalid header tag")
         mem_id = (header.flags & cls.ROM_MEM_DEVICE_ID_MASK) >> cls.ROM_MEM_DEVICE_ID_SHIFT
         return cls(header.address, mem_id, header.count, header.data, header.flags)
 
 
-class VersionCheckType(Enum):
+class VersionCheckType(SpsdkEnum):
     """Select type of the version check: either secure or non-secure firmware to be checked."""
 
-    SECURE_VERSION = 0
-    NON_SECURE_VERSION = 1
+    SECURE_VERSION = (0, "SECURE_VERSION")
+    NON_SECURE_VERSION = (1, "NON_SECURE_VERSION")
 
 
 class CmdVersionCheck(CmdBaseClass):
@@ -867,15 +861,15 @@ class CmdVersionCheck(CmdBaseClass):
         :raises SPSDKError: If invalid version check type
         """
         super().__init__(EnumCmdTag.FW_VERSION_CHECK)
-        if ver_type not in VersionCheckType.tags():
+        if ver_type not in VersionCheckType:
             raise SPSDKError("Invalid version check type")
-        self.header.address = ver_type
+        self.header.address = ver_type.tag
         self.header.count = version
 
     @property
     def type(self) -> VersionCheckType:
         """Return type of the check version, see VersionCheckType enumeration."""
-        return VersionCheckType.from_int(self.header.address)
+        return VersionCheckType.from_tag(self.header.address)
 
     @property
     def version(self) -> int:
@@ -884,25 +878,24 @@ class CmdVersionCheck(CmdBaseClass):
 
     def __str__(self) -> str:
         return (
-            f"CVER: Type={VersionCheckType.name(self.type)}, Version={str(self.version)}, "
+            f"CVER: Type={self.type.label}, Version={str(self.version)}, "
             f"Flags=0x{self.header.flags:08X}"
         )
 
     @classmethod
-    def parse(cls, data: bytes, offset: int = 0) -> "CmdVersionCheck":
+    def parse(cls, data: bytes) -> Self:
         """Parse command from bytes.
 
         :param data: Input data as bytes
-        :param offset: The offset of input data
         :return: parsed command object
         :raises SPSDKError: If incorrect header tag
         """
-        header = CmdHeader.parse(data, offset)
+        header = CmdHeader.parse(data)
         if header.tag != EnumCmdTag.FW_VERSION_CHECK:
             raise SPSDKError("Invalid header tag")
-        ver_type = VersionCheckType.from_int(header.address)
+        ver_type = VersionCheckType.from_tag(header.address)
         version = header.count
-        return CmdVersionCheck(ver_type, version)
+        return cls(ver_type, version)
 
 
 class CmdKeyStoreBackupRestore(CmdBaseClass):
@@ -934,10 +927,10 @@ class CmdKeyStoreBackupRestore(CmdBaseClass):
         if address < 0 or address > 0xFFFFFFFF:
             raise SPSDKError("Invalid address")
         self.header.address = address
-        if controller_id < 0 or controller_id > 0xFF:
+        if controller_id.tag < 0 or controller_id.tag > 0xFF:
             raise SPSDKError("Invalid ID of memory")
         self.header.flags = (self.header.flags & ~self.ROM_MEM_DEVICE_ID_MASK) | (
-            (controller_id << self.ROM_MEM_DEVICE_ID_SHIFT) & self.ROM_MEM_DEVICE_ID_MASK
+            (controller_id.tag << self.ROM_MEM_DEVICE_ID_SHIFT) & self.ROM_MEM_DEVICE_ID_MASK
         )
         self.header.count = (
             4  # this is useless, but it is kept for backward compatibility with elftosb
@@ -954,20 +947,19 @@ class CmdKeyStoreBackupRestore(CmdBaseClass):
         return (self.header.flags & self.ROM_MEM_DEVICE_ID_MASK) >> self.ROM_MEM_DEVICE_ID_SHIFT
 
     @classmethod
-    def parse(cls, data: bytes, offset: int = 0) -> "CmdKeyStoreBackupRestore":
+    def parse(cls, data: bytes) -> Self:
         """Parse command from bytes.
 
         :param data: Input data as bytes
-        :param offset: The offset of input data
         :return: CmdKeyStoreBackupRestore object
         :raises SPSDKError: When there is invalid header tag
         """
-        header = CmdHeader.parse(data, offset)
+        header = CmdHeader.parse(data)
         if header.tag != cls.cmd_id():
             raise SPSDKError("Invalid header tag")
         address = header.address
         controller_id = (header.flags & cls.ROM_MEM_DEVICE_ID_MASK) >> cls.ROM_MEM_DEVICE_ID_SHIFT
-        return cls(address, controller_id)  # type: ignore
+        return cls(address, ExtMemId.from_tag(controller_id))
 
 
 class CmdKeyStoreBackup(CmdKeyStoreBackupRestore):
@@ -991,7 +983,7 @@ class CmdKeyStoreRestore(CmdKeyStoreBackupRestore):
 ########################################################################################################################
 # Command parser from binary format
 ########################################################################################################################
-_CMD_CLASS: Mapping[int, Type[CmdBaseClass]] = {
+_CMD_CLASS: Mapping[EnumCmdTag, Type[CmdBaseClass]] = {
     EnumCmdTag.NOP: CmdNop,
     EnumCmdTag.TAG: CmdTag,
     EnumCmdTag.LOAD: CmdLoad,
@@ -1008,18 +1000,18 @@ _CMD_CLASS: Mapping[int, Type[CmdBaseClass]] = {
 }
 
 
-def parse_command(data: bytes, offset: int = 0) -> CmdBaseClass:
+def parse_command(data: bytes) -> CmdBaseClass:
     """Parse SB 2.x command from bytes.
 
     :param data: Input data as bytes
-    :param offset: The offset of input data to start parsing
     :return: parsed command object
     :raises SPSDKError: Raised when there is unsupported command provided
     """
-    header_tag = data[offset + 1]
-    if header_tag not in _CMD_CLASS:
-        raise SPSDKError(f"Unsupported command: {str(header_tag)}")
-    return _CMD_CLASS[header_tag].parse(data, offset)
+    header_tag = data[1]
+    for cmd_tag, cmd in _CMD_CLASS.items():
+        if cmd_tag.tag == header_tag:
+            return cmd.parse(data)
+    raise SPSDKError(f"Unsupported command: {str(header_tag)}")
 
 
 def get_device_id(mem_id: int) -> int:
