@@ -2,14 +2,14 @@
 # -*- coding: UTF-8 -*-
 #
 # Copyright 2017-2018 Martin Olejar
-# Copyright 2019-2021 NXP
+# Copyright 2019-2024 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
 """Exceptions used in the SDP module."""
 
-from .error_codes import StatusCode
-from ..exceptions import SPSDKError
+from spsdk.exceptions import SPSDKConnectionError, SPSDKError
+from spsdk.sdp.error_codes import StatusCode
 
 
 ########################################################################################################################
@@ -35,13 +35,17 @@ class SdpCommandError(SdpError):
         super().__init__()
         self.cmd_name = cmd
         self.error_value = value
-        self.description = StatusCode.desc(value, f"Unknown Error 0x{value:08X}")
+        self.description = (
+            StatusCode.from_tag(value).description
+            if value in StatusCode.tags()
+            else f"Unknown Error 0x{value:08X}"
+        )
 
     def __str__(self) -> str:
         return self.fmt.format(cmd_name=self.cmd_name, description=self.description)
 
 
-class SdpConnectionError(SdpError):
+class SdpConnectionError(SPSDKConnectionError, SdpError):
     """SDP Module: Connection Exception."""
 
     fmt = "SDP: Connection issue -> {description}"
