@@ -1,23 +1,18 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2020-2023 NXP
+# Copyright 2020-2024 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
 """SerialBuspalProxy serves as patch replacement for serial.Serial class."""
 
 import logging
-from typing import (  # pylint: disable=unused-import  # Type is necessary for Mypy
-    Dict,
-    Optional,
-    Type,
-)
+from typing import Optional, Type  # pylint: disable=unused-import  # Type is necessary for Mypy
 
-from spsdk.mboot.interfaces.buspal_i2c import I2cModeCommand
-from spsdk.mboot.interfaces.buspal_spi import SpiModeCommand
-
-from .serial_proxy import SerialProxy
+from spsdk.exceptions import SPSDKError
+from spsdk.mboot.interfaces.buspal import I2cModeCommand, SpiModeCommand
+from spsdk.utils.serial_proxy import SerialProxy
 
 logger = logging.getLogger(__name__)
 
@@ -32,20 +27,20 @@ class SerialBuspalProxy(SerialProxy):
     frame_header: int
 
     @classmethod
-    def init_buspal_proxy(cls, target: str, data: Dict[bytes, bytes]) -> "Type[SerialProxy]":
+    def init_buspal_proxy(cls, target: str, data: dict[bytes, bytes]) -> "Type[SerialProxy]":
         """Initialized response dictionary of write and read bytes.
 
         :param target: BUSPAL target type
         :param data: Dictionary of write and read bytes
         :return: SerialProxy class with configured data
-        :raise AttributeError: target not supported
+        :raises SPSDKError: target not supported
         """
         if target == "i2c":
             cls.frame_header = I2cModeCommand.write_then_read.value
         elif target == "spi":
             cls.frame_header = SpiModeCommand.write_then_read.value
         else:
-            raise AttributeError(f"Target {target} not supported")
+            raise SPSDKError(f"Target {target} not supported")
         return super().init_proxy(data)
 
     def __init__(self, port: str, timeout: int, baudrate: int, write_timeout: Optional[int] = None):
