@@ -1,14 +1,17 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2019-2021 NXP
+# Copyright 2019-2024 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
 """Module defining a USB filtering class."""
 import platform
 import re
-from typing import Any, Dict, Tuple
+from typing import Any, Optional
+
+from spsdk.utils.database import UsbId
+from spsdk.utils.misc import get_hash
 
 
 class USBDeviceFilter:
@@ -23,7 +26,7 @@ class USBDeviceFilter:
     Hex number must be preceded by 0x or 0X. Number of characters after 0x is
     1 - 4. Mixed upper & lower case letters is allowed. e.g. "0xaB12", "0XAB12",
     "0x1", "0x0001".
-    The decimal number is restrictred only to have 1 - 5 digits, e.g. "65535"
+    The decimal number is restricted only to have 1 - 5 digits, e.g. "65535"
     It's allowed to set the USB filter ID to decimal number "99999", however, as
     the USB VID number is four-byte hex number (max value is 65535), this will
     lead to zero results. Leading zeros are not allowed e.g. 0001. This will
@@ -73,7 +76,7 @@ class USBDeviceFilter:
 
     def __init__(
         self,
-        usb_id: str = None,
+        usb_id: Optional[str] = None,
         search_by_pid: bool = False,
     ):
         """Initialize the USB Device Filtering.
@@ -84,7 +87,7 @@ class USBDeviceFilter:
         self.usb_id = usb_id
         self.search_by_pid = search_by_pid
 
-    def compare(self, usb_device_object: Any) -> bool:
+    def compare(self, usb_device_object: dict[str, Any]) -> bool:
         """Compares the internal `usb_id` with provided `usb_device_object`.
 
         The provided USB ID during initialization may be VID or PID, VID/PID pair,
@@ -94,43 +97,34 @@ class USBDeviceFilter:
 
         :return: True on match, False otherwise
         """
-        vendor_id = usb_device_object["vendor_id"]
-        product_id = usb_device_object["product_id"]
-
-        # the Libusbsio/HID_API holds the path as bytes, so we convert it to string
-        usb_path = usb_device_object["path"].decode("utf-8")
-
-        if platform.system() == "Windows":
-            # On WIN, the user has an instance ID (+ the tool expects following format):
-            # 'HID\\VID_1FC9&PID_0130\\6&3B9928A5&0&0000'
-            # However, the path has following format:
-            # '\\\\?\\hid#vid_1fc9&pid_0130#6&1625c75b&0&0000#{4d1e55b2-f16f-11cf-88cb-001111000030}'
-            # There is a pattern, which matches, so we take the path and modify it
-            # to match the instance ID format. We convert the path to upper case and
-            # replace the hash sign with backslash
-            # usb_path = usb_path.upper()
-            usb_path = usb_path.replace("#", "\\")
-
-        if platform.system() == "Linux":
-            # The user input is expected in form of <dec_num>#<dec_num>. So we
-            # convert the path returned by Libusbsio/HID_API into this form so we can
-            # compare it. Alternatively, the input is the real device path,
-            # like '/dev/hidraw0' - in this case, just leave it as it is.
-            nums = usb_path.split(":")
-            if len(nums) >= 2:
-                usb_path = str.format("{}#{}", int(nums[0], 16), int(nums[1], 16))
-
         # Determine, whether given device matches one of the expected criterion
         if self.usb_id is None:
             return True
 
-        if self._is_path(usb_path=usb_path):
-            return True
+        vendor_id = usb_device_object.get("vendor_id")
+        product_id = usb_device_object.get("product_id")
+        serial_number = usb_device_object.get("serial_number")
+        device_name = usb_device_object.get("device_name")
+        # the Libusbsio/HID_API holds the path as bytes, so we convert it to string
+        usb_path_raw = usb_device_object.get("path")
+
+        if usb_path_raw:
+            if self.usb_id == get_hash(usb_path_raw):
+                return True
+            usb_path = self.convert_usb_path(usb_path_raw)
+            if self._is_path(usb_path=usb_path):
+                return True
 
         if self._is_vid_or_pid(vid=vendor_id, pid=product_id):
             return True
 
-        if self._is_vid_pid(vid=vendor_id, pid=product_id):
+        if vendor_id and product_id and self._is_vid_pid(vid=vendor_id, pid=product_id):
+            return True
+
+        if serial_number and self.usb_id.casefold() == serial_number.casefold():
+            return True
+
+        if device_name and self.usb_id.casefold() == device_name.casefold():
             return True
 
         return False
@@ -146,7 +140,7 @@ class USBDeviceFilter:
         """
         # we check the len of usb_id, because usb_id = "" is considered
         # to be always in the string returning True, which is not expected
-        # behaviour
+        # behavior
         # the provided usb string id fully matches the instance ID
         usb_id = self.usb_id or ""
         if usb_id.casefold() in usb_path.casefold() and len(usb_id) > 0:
@@ -154,7 +148,7 @@ class USBDeviceFilter:
 
         return False
 
-    def _is_vid_or_pid(self, vid: int, pid: int) -> bool:
+    def _is_vid_or_pid(self, vid: Optional[int], pid: Optional[int]) -> bool:
         # match anything starting with 0x or 0X followed by 0-9 or a-f or
         # match either 0 or decimal number not starting with zero
         # this regex is the same for vid and pid => xid
@@ -162,17 +156,17 @@ class USBDeviceFilter:
         usb_id = self.usb_id or ""
         if re.fullmatch(xid_regex, usb_id) is not None:
             # the string corresponds to the vid/pid specification, check a match
-            if self.search_by_pid:
+            if self.search_by_pid and pid:
                 if int(usb_id, 0) == pid:
                     return True
-            else:
+            elif vid:
                 if int(usb_id, 0) == vid:
                     return True
 
         return False
 
     def _is_vid_pid(self, vid: int, pid: int) -> bool:
-        """If usb_id corresponds to VID/PID pair, comapres it with provided vid/pid.
+        """If usb_id corresponds to VID/PID pair, compares it with provided vid/pid.
 
         :param vid: vendor ID to compare.
         :param pid: product ID to compare.
@@ -191,6 +185,47 @@ class USBDeviceFilter:
 
         return False
 
+    @staticmethod
+    def convert_usb_path(hid_api_usb_path: bytes) -> str:
+        """Converts the Libusbsio/HID_API path into string, which can be observed from OS.
+
+        DESIGN REMARK: this function is not part of the USBLogicalDevice, as the
+        class intention is to be just a simple container. But to help the class
+        to get the required inputs, this helper method has been provided. Additionally,
+        this method relies on the fact that the provided path comes from the Libusbsio/HID_API.
+        This method will most probably fail or provide improper results in case
+        path from different USB API is provided.
+
+        :param hid_api_usb_path: USB device path from Libusbsio/HID_API
+        :return: Libusbsio/HID_API path converted for given platform
+        """
+        if platform.system() == "Windows":
+            device_manager_path = hid_api_usb_path.decode("utf-8").upper()
+            device_manager_path = device_manager_path.replace("#", "\\")
+            result = re.search(r"\\\\\?\\(.+?)\\{", device_manager_path)
+            if result:
+                device_manager_path = result.group(1)
+
+            return device_manager_path
+
+        if platform.system() == "Linux":
+            # we expect the path in form of <bus>#<device>, Libusbsio/HID_API returns
+            # <bus>:<device>:<interface>
+            linux_path = hid_api_usb_path.decode("utf-8")
+            linux_path_parts = linux_path.split(":")
+
+            if len(linux_path_parts) > 1:
+                linux_path = str.format(
+                    "{}#{}", int(linux_path_parts[0], 16), int(linux_path_parts[1], 16)
+                )
+
+            return linux_path
+
+        if platform.system() == "Darwin":
+            return hid_api_usb_path.decode("utf-8")
+
+        return ""
+
 
 class NXPUSBDeviceFilter(USBDeviceFilter):
     """NXP Device Filtering class.
@@ -202,12 +237,12 @@ class NXPUSBDeviceFilter(USBDeviceFilter):
     a single number is provided, we expect that VID is out of range NXP_VIDS.
     """
 
-    NXP_VIDS = [0x1FC9, 0x15A2]
+    NXP_VIDS = [0x1FC9, 0x15A2, 0x0471, 0x0D28]
 
     def __init__(
         self,
-        usb_id: str = None,
-        nxp_device_names: Dict[str, Tuple[int, int]] = None,
+        usb_id: Optional[str] = None,
+        nxp_device_names: Optional[dict[str, list[UsbId]]] = None,
     ):
         """Initialize the USB Device Filtering.
 
@@ -230,26 +265,28 @@ class NXPUSBDeviceFilter(USBDeviceFilter):
         vendor_id = usb_device_object["vendor_id"]
         product_id = usb_device_object["product_id"]
 
-        is_valid_id = super().compare(usb_device_object=usb_device_object)
-
-        if is_valid_id:
-            return True
-
-        if self._is_nxp_device_name(vendor_id, product_id):
-            return True
-
-        return False
-
-    def _is_vid_or_pid(self, vid: int, pid: int) -> bool:
-        if vid in NXPUSBDeviceFilter.NXP_VIDS:
-            if True == super()._is_vid_or_pid(vid, pid):
+        if self.usb_id:
+            if super().compare(usb_device_object=usb_device_object):
                 return True
 
+            return self._is_nxp_device_name(vendor_id, product_id)
+
+        return self._is_nxp_device(vendor_id)
+
+    def _is_vid_or_pid(self, vid: Optional[int], pid: Optional[int]) -> bool:
+        if vid and vid in NXPUSBDeviceFilter.NXP_VIDS:
+            return super()._is_vid_or_pid(vid, pid)
+
         return False
 
     def _is_nxp_device_name(self, vid: int, pid: int) -> bool:
+        assert isinstance(self.usb_id, str)
         if self.usb_id in self.nxp_device_names:
-            vendor_id, product_id = self.nxp_device_names[self.usb_id]
-            if vendor_id == vid and product_id == pid:
-                return True
+            for usb_cfg in self.nxp_device_names[self.usb_id]:
+                if usb_cfg.vid == vid and usb_cfg.pid == pid:
+                    return True
         return False
+
+    @staticmethod
+    def _is_nxp_device(vid: int) -> bool:
+        return vid in NXPUSBDeviceFilter.NXP_VIDS
